# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class BillingScheduleSummary(object):
    """
    Billing schedule details related to Subscription Id
    """

    #: A constant which can be used with the invoice_status property of a BillingScheduleSummary.
    #: This constant has a value of "INVOICED"
    INVOICE_STATUS_INVOICED = "INVOICED"

    #: A constant which can be used with the invoice_status property of a BillingScheduleSummary.
    #: This constant has a value of "NOT_INVOICED"
    INVOICE_STATUS_NOT_INVOICED = "NOT_INVOICED"

    def __init__(self, **kwargs):
        """
        Initializes a new BillingScheduleSummary object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param time_start:
            The value to assign to the time_start property of this BillingScheduleSummary.
        :type time_start: datetime

        :param time_end:
            The value to assign to the time_end property of this BillingScheduleSummary.
        :type time_end: datetime

        :param time_invoicing:
            The value to assign to the time_invoicing property of this BillingScheduleSummary.
        :type time_invoicing: datetime

        :param invoice_status:
            The value to assign to the invoice_status property of this BillingScheduleSummary.
            Allowed values for this property are: "INVOICED", "NOT_INVOICED", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type invoice_status: str

        :param quantity:
            The value to assign to the quantity property of this BillingScheduleSummary.
        :type quantity: str

        :param net_unit_price:
            The value to assign to the net_unit_price property of this BillingScheduleSummary.
        :type net_unit_price: str

        :param amount:
            The value to assign to the amount property of this BillingScheduleSummary.
        :type amount: str

        :param billing_frequency:
            The value to assign to the billing_frequency property of this BillingScheduleSummary.
        :type billing_frequency: str

        :param ar_invoice_number:
            The value to assign to the ar_invoice_number property of this BillingScheduleSummary.
        :type ar_invoice_number: str

        :param ar_customer_transaction_id:
            The value to assign to the ar_customer_transaction_id property of this BillingScheduleSummary.
        :type ar_customer_transaction_id: str

        :param order_number:
            The value to assign to the order_number property of this BillingScheduleSummary.
        :type order_number: str

        :param product:
            The value to assign to the product property of this BillingScheduleSummary.
        :type product: oci.osub_billing_schedule.models.Product

        """
        self.swagger_types = {
            'time_start': 'datetime',
            'time_end': 'datetime',
            'time_invoicing': 'datetime',
            'invoice_status': 'str',
            'quantity': 'str',
            'net_unit_price': 'str',
            'amount': 'str',
            'billing_frequency': 'str',
            'ar_invoice_number': 'str',
            'ar_customer_transaction_id': 'str',
            'order_number': 'str',
            'product': 'Product'
        }

        self.attribute_map = {
            'time_start': 'timeStart',
            'time_end': 'timeEnd',
            'time_invoicing': 'timeInvoicing',
            'invoice_status': 'invoiceStatus',
            'quantity': 'quantity',
            'net_unit_price': 'netUnitPrice',
            'amount': 'amount',
            'billing_frequency': 'billingFrequency',
            'ar_invoice_number': 'arInvoiceNumber',
            'ar_customer_transaction_id': 'arCustomerTransactionId',
            'order_number': 'orderNumber',
            'product': 'product'
        }

        self._time_start = None
        self._time_end = None
        self._time_invoicing = None
        self._invoice_status = None
        self._quantity = None
        self._net_unit_price = None
        self._amount = None
        self._billing_frequency = None
        self._ar_invoice_number = None
        self._ar_customer_transaction_id = None
        self._order_number = None
        self._product = None

    @property
    def time_start(self):
        """
        Gets the time_start of this BillingScheduleSummary.
        Billing schedule start date


        :return: The time_start of this BillingScheduleSummary.
        :rtype: datetime
        """
        return self._time_start

    @time_start.setter
    def time_start(self, time_start):
        """
        Sets the time_start of this BillingScheduleSummary.
        Billing schedule start date


        :param time_start: The time_start of this BillingScheduleSummary.
        :type: datetime
        """
        self._time_start = time_start

    @property
    def time_end(self):
        """
        Gets the time_end of this BillingScheduleSummary.
        Billing schedule end date


        :return: The time_end of this BillingScheduleSummary.
        :rtype: datetime
        """
        return self._time_end

    @time_end.setter
    def time_end(self, time_end):
        """
        Sets the time_end of this BillingScheduleSummary.
        Billing schedule end date


        :param time_end: The time_end of this BillingScheduleSummary.
        :type: datetime
        """
        self._time_end = time_end

    @property
    def time_invoicing(self):
        """
        Gets the time_invoicing of this BillingScheduleSummary.
        Billing schedule invoicing date


        :return: The time_invoicing of this BillingScheduleSummary.
        :rtype: datetime
        """
        return self._time_invoicing

    @time_invoicing.setter
    def time_invoicing(self, time_invoicing):
        """
        Sets the time_invoicing of this BillingScheduleSummary.
        Billing schedule invoicing date


        :param time_invoicing: The time_invoicing of this BillingScheduleSummary.
        :type: datetime
        """
        self._time_invoicing = time_invoicing

    @property
    def invoice_status(self):
        """
        Gets the invoice_status of this BillingScheduleSummary.
        Billing schedule invoice status

        Allowed values for this property are: "INVOICED", "NOT_INVOICED", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The invoice_status of this BillingScheduleSummary.
        :rtype: str
        """
        return self._invoice_status

    @invoice_status.setter
    def invoice_status(self, invoice_status):
        """
        Sets the invoice_status of this BillingScheduleSummary.
        Billing schedule invoice status


        :param invoice_status: The invoice_status of this BillingScheduleSummary.
        :type: str
        """
        allowed_values = ["INVOICED", "NOT_INVOICED"]
        if not value_allowed_none_or_none_sentinel(invoice_status, allowed_values):
            invoice_status = 'UNKNOWN_ENUM_VALUE'
        self._invoice_status = invoice_status

    @property
    def quantity(self):
        """
        Gets the quantity of this BillingScheduleSummary.
        Billing schedule quantity


        :return: The quantity of this BillingScheduleSummary.
        :rtype: str
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """
        Sets the quantity of this BillingScheduleSummary.
        Billing schedule quantity


        :param quantity: The quantity of this BillingScheduleSummary.
        :type: str
        """
        self._quantity = quantity

    @property
    def net_unit_price(self):
        """
        Gets the net_unit_price of this BillingScheduleSummary.
        Billing schedule net unit price


        :return: The net_unit_price of this BillingScheduleSummary.
        :rtype: str
        """
        return self._net_unit_price

    @net_unit_price.setter
    def net_unit_price(self, net_unit_price):
        """
        Sets the net_unit_price of this BillingScheduleSummary.
        Billing schedule net unit price


        :param net_unit_price: The net_unit_price of this BillingScheduleSummary.
        :type: str
        """
        self._net_unit_price = net_unit_price

    @property
    def amount(self):
        """
        Gets the amount of this BillingScheduleSummary.
        Billing schedule line net amount


        :return: The amount of this BillingScheduleSummary.
        :rtype: str
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """
        Sets the amount of this BillingScheduleSummary.
        Billing schedule line net amount


        :param amount: The amount of this BillingScheduleSummary.
        :type: str
        """
        self._amount = amount

    @property
    def billing_frequency(self):
        """
        Gets the billing_frequency of this BillingScheduleSummary.
        Billing frequency


        :return: The billing_frequency of this BillingScheduleSummary.
        :rtype: str
        """
        return self._billing_frequency

    @billing_frequency.setter
    def billing_frequency(self, billing_frequency):
        """
        Sets the billing_frequency of this BillingScheduleSummary.
        Billing frequency


        :param billing_frequency: The billing_frequency of this BillingScheduleSummary.
        :type: str
        """
        self._billing_frequency = billing_frequency

    @property
    def ar_invoice_number(self):
        """
        Gets the ar_invoice_number of this BillingScheduleSummary.
        Indicates the associated AR Invoice Number


        :return: The ar_invoice_number of this BillingScheduleSummary.
        :rtype: str
        """
        return self._ar_invoice_number

    @ar_invoice_number.setter
    def ar_invoice_number(self, ar_invoice_number):
        """
        Sets the ar_invoice_number of this BillingScheduleSummary.
        Indicates the associated AR Invoice Number


        :param ar_invoice_number: The ar_invoice_number of this BillingScheduleSummary.
        :type: str
        """
        self._ar_invoice_number = ar_invoice_number

    @property
    def ar_customer_transaction_id(self):
        """
        Gets the ar_customer_transaction_id of this BillingScheduleSummary.
        Indicates the associated AR Customer transaction id a unique identifier existing on AR.


        :return: The ar_customer_transaction_id of this BillingScheduleSummary.
        :rtype: str
        """
        return self._ar_customer_transaction_id

    @ar_customer_transaction_id.setter
    def ar_customer_transaction_id(self, ar_customer_transaction_id):
        """
        Sets the ar_customer_transaction_id of this BillingScheduleSummary.
        Indicates the associated AR Customer transaction id a unique identifier existing on AR.


        :param ar_customer_transaction_id: The ar_customer_transaction_id of this BillingScheduleSummary.
        :type: str
        """
        self._ar_customer_transaction_id = ar_customer_transaction_id

    @property
    def order_number(self):
        """
        Gets the order_number of this BillingScheduleSummary.
        Order number associated with the Subscribed Service


        :return: The order_number of this BillingScheduleSummary.
        :rtype: str
        """
        return self._order_number

    @order_number.setter
    def order_number(self, order_number):
        """
        Sets the order_number of this BillingScheduleSummary.
        Order number associated with the Subscribed Service


        :param order_number: The order_number of this BillingScheduleSummary.
        :type: str
        """
        self._order_number = order_number

    @property
    def product(self):
        """
        Gets the product of this BillingScheduleSummary.

        :return: The product of this BillingScheduleSummary.
        :rtype: oci.osub_billing_schedule.models.Product
        """
        return self._product

    @product.setter
    def product(self, product):
        """
        Sets the product of this BillingScheduleSummary.

        :param product: The product of this BillingScheduleSummary.
        :type: oci.osub_billing_schedule.models.Product
        """
        self._product = product

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class AdvisorReportLocationDetails(object):
    """
    Details to access Pre-Migration Advisor report.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new AdvisorReportLocationDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param object_storage_details:
            The value to assign to the object_storage_details property of this AdvisorReportLocationDetails.
        :type object_storage_details: oci.database_migration.models.AdvisorReportBucketDetails

        :param location_in_source:
            The value to assign to the location_in_source property of this AdvisorReportLocationDetails.
        :type location_in_source: str

        """
        self.swagger_types = {
            'object_storage_details': 'AdvisorReportBucketDetails',
            'location_in_source': 'str'
        }

        self.attribute_map = {
            'object_storage_details': 'objectStorageDetails',
            'location_in_source': 'locationInSource'
        }

        self._object_storage_details = None
        self._location_in_source = None

    @property
    def object_storage_details(self):
        """
        Gets the object_storage_details of this AdvisorReportLocationDetails.

        :return: The object_storage_details of this AdvisorReportLocationDetails.
        :rtype: oci.database_migration.models.AdvisorReportBucketDetails
        """
        return self._object_storage_details

    @object_storage_details.setter
    def object_storage_details(self, object_storage_details):
        """
        Sets the object_storage_details of this AdvisorReportLocationDetails.

        :param object_storage_details: The object_storage_details of this AdvisorReportLocationDetails.
        :type: oci.database_migration.models.AdvisorReportBucketDetails
        """
        self._object_storage_details = object_storage_details

    @property
    def location_in_source(self):
        """
        Gets the location_in_source of this AdvisorReportLocationDetails.
        Path in the Source Registered Connection where the Pre-Migration advisor report can be accessed.


        :return: The location_in_source of this AdvisorReportLocationDetails.
        :rtype: str
        """
        return self._location_in_source

    @location_in_source.setter
    def location_in_source(self, location_in_source):
        """
        Sets the location_in_source of this AdvisorReportLocationDetails.
        Path in the Source Registered Connection where the Pre-Migration advisor report can be accessed.


        :param location_in_source: The location_in_source of this AdvisorReportLocationDetails.
        :type: str
        """
        self._location_in_source = location_in_source

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class DataObjectQueryTimeFilters(object):
    """
    Time filters to be applied in the data object query.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new DataObjectQueryTimeFilters object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param time_period:
            The value to assign to the time_period property of this DataObjectQueryTimeFilters.
        :type time_period: str

        :param time_start:
            The value to assign to the time_start property of this DataObjectQueryTimeFilters.
        :type time_start: datetime

        :param time_end:
            The value to assign to the time_end property of this DataObjectQueryTimeFilters.
        :type time_end: datetime

        """
        self.swagger_types = {
            'time_period': 'str',
            'time_start': 'datetime',
            'time_end': 'datetime'
        }

        self.attribute_map = {
            'time_period': 'timePeriod',
            'time_start': 'timeStart',
            'time_end': 'timeEnd'
        }

        self._time_period = None
        self._time_start = None
        self._time_end = None

    @property
    def time_period(self):
        """
        Gets the time_period of this DataObjectQueryTimeFilters.
        Specify time period in ISO 8601 format with respect to current time.
        Default is last 30 days represented by P30D.
        If timePeriod is specified, then timeStart and timeEnd will be ignored.
        Examples: P90D (last 90 days), P4W (last 4 weeks), P2M (last 2 months), P1Y (last 12 months).


        :return: The time_period of this DataObjectQueryTimeFilters.
        :rtype: str
        """
        return self._time_period

    @time_period.setter
    def time_period(self, time_period):
        """
        Sets the time_period of this DataObjectQueryTimeFilters.
        Specify time period in ISO 8601 format with respect to current time.
        Default is last 30 days represented by P30D.
        If timePeriod is specified, then timeStart and timeEnd will be ignored.
        Examples: P90D (last 90 days), P4W (last 4 weeks), P2M (last 2 months), P1Y (last 12 months).


        :param time_period: The time_period of this DataObjectQueryTimeFilters.
        :type: str
        """
        self._time_period = time_period

    @property
    def time_start(self):
        """
        Gets the time_start of this DataObjectQueryTimeFilters.
        Start time in UTC in RFC3339 formatted datetime string. Example: 2021-10-30T00:00:00.000Z.
        timeStart and timeEnd are used together. If timePeriod is specified, this parameter is ignored.


        :return: The time_start of this DataObjectQueryTimeFilters.
        :rtype: datetime
        """
        return self._time_start

    @time_start.setter
    def time_start(self, time_start):
        """
        Sets the time_start of this DataObjectQueryTimeFilters.
        Start time in UTC in RFC3339 formatted datetime string. Example: 2021-10-30T00:00:00.000Z.
        timeStart and timeEnd are used together. If timePeriod is specified, this parameter is ignored.


        :param time_start: The time_start of this DataObjectQueryTimeFilters.
        :type: datetime
        """
        self._time_start = time_start

    @property
    def time_end(self):
        """
        Gets the time_end of this DataObjectQueryTimeFilters.
        End time in UTC in RFC3339 formatted datetime string. Example: 2021-10-30T00:00:00.000Z.
        timeStart and timeEnd are used together. If timePeriod is specified, this parameter is ignored.
        If timeEnd is not specified, current time is used as timeEnd.


        :return: The time_end of this DataObjectQueryTimeFilters.
        :rtype: datetime
        """
        return self._time_end

    @time_end.setter
    def time_end(self, time_end):
        """
        Sets the time_end of this DataObjectQueryTimeFilters.
        End time in UTC in RFC3339 formatted datetime string. Example: 2021-10-30T00:00:00.000Z.
        timeStart and timeEnd are used together. If timePeriod is specified, this parameter is ignored.
        If timeEnd is not specified, current time is used as timeEnd.


        :param time_end: The time_end of this DataObjectQueryTimeFilters.
        :type: datetime
        """
        self._time_end = time_end

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class DiscoveryDetails(object):
    """
    The request of DiscoveryJob Resource details.
    """

    #: A constant which can be used with the resource_type property of a DiscoveryDetails.
    #: This constant has a value of "WEBLOGIC_DOMAIN"
    RESOURCE_TYPE_WEBLOGIC_DOMAIN = "WEBLOGIC_DOMAIN"

    #: A constant which can be used with the resource_type property of a DiscoveryDetails.
    #: This constant has a value of "EBS_INSTANCE"
    RESOURCE_TYPE_EBS_INSTANCE = "EBS_INSTANCE"

    #: A constant which can be used with the resource_type property of a DiscoveryDetails.
    #: This constant has a value of "ORACLE_DATABASE"
    RESOURCE_TYPE_ORACLE_DATABASE = "ORACLE_DATABASE"

    def __init__(self, **kwargs):
        """
        Initializes a new DiscoveryDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param agent_id:
            The value to assign to the agent_id property of this DiscoveryDetails.
        :type agent_id: str

        :param resource_type:
            The value to assign to the resource_type property of this DiscoveryDetails.
            Allowed values for this property are: "WEBLOGIC_DOMAIN", "EBS_INSTANCE", "ORACLE_DATABASE", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type resource_type: str

        :param resource_name:
            The value to assign to the resource_name property of this DiscoveryDetails.
        :type resource_name: str

        :param properties:
            The value to assign to the properties property of this DiscoveryDetails.
        :type properties: oci.stack_monitoring.models.PropertyDetails

        :param credentials:
            The value to assign to the credentials property of this DiscoveryDetails.
        :type credentials: oci.stack_monitoring.models.CredentialCollection

        :param tags:
            The value to assign to the tags property of this DiscoveryDetails.
        :type tags: oci.stack_monitoring.models.PropertyDetails

        """
        self.swagger_types = {
            'agent_id': 'str',
            'resource_type': 'str',
            'resource_name': 'str',
            'properties': 'PropertyDetails',
            'credentials': 'CredentialCollection',
            'tags': 'PropertyDetails'
        }

        self.attribute_map = {
            'agent_id': 'agentId',
            'resource_type': 'resourceType',
            'resource_name': 'resourceName',
            'properties': 'properties',
            'credentials': 'credentials',
            'tags': 'tags'
        }

        self._agent_id = None
        self._resource_type = None
        self._resource_name = None
        self._properties = None
        self._credentials = None
        self._tags = None

    @property
    def agent_id(self):
        """
        **[Required]** Gets the agent_id of this DiscoveryDetails.
        The OCID of Management Agent


        :return: The agent_id of this DiscoveryDetails.
        :rtype: str
        """
        return self._agent_id

    @agent_id.setter
    def agent_id(self, agent_id):
        """
        Sets the agent_id of this DiscoveryDetails.
        The OCID of Management Agent


        :param agent_id: The agent_id of this DiscoveryDetails.
        :type: str
        """
        self._agent_id = agent_id

    @property
    def resource_type(self):
        """
        **[Required]** Gets the resource_type of this DiscoveryDetails.
        Resource Type.

        Allowed values for this property are: "WEBLOGIC_DOMAIN", "EBS_INSTANCE", "ORACLE_DATABASE", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The resource_type of this DiscoveryDetails.
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """
        Sets the resource_type of this DiscoveryDetails.
        Resource Type.


        :param resource_type: The resource_type of this DiscoveryDetails.
        :type: str
        """
        allowed_values = ["WEBLOGIC_DOMAIN", "EBS_INSTANCE", "ORACLE_DATABASE"]
        if not value_allowed_none_or_none_sentinel(resource_type, allowed_values):
            resource_type = 'UNKNOWN_ENUM_VALUE'
        self._resource_type = resource_type

    @property
    def resource_name(self):
        """
        **[Required]** Gets the resource_name of this DiscoveryDetails.
        The Name of resource type


        :return: The resource_name of this DiscoveryDetails.
        :rtype: str
        """
        return self._resource_name

    @resource_name.setter
    def resource_name(self, resource_name):
        """
        Sets the resource_name of this DiscoveryDetails.
        The Name of resource type


        :param resource_name: The resource_name of this DiscoveryDetails.
        :type: str
        """
        self._resource_name = resource_name

    @property
    def properties(self):
        """
        **[Required]** Gets the properties of this DiscoveryDetails.

        :return: The properties of this DiscoveryDetails.
        :rtype: oci.stack_monitoring.models.PropertyDetails
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """
        Sets the properties of this DiscoveryDetails.

        :param properties: The properties of this DiscoveryDetails.
        :type: oci.stack_monitoring.models.PropertyDetails
        """
        self._properties = properties

    @property
    def credentials(self):
        """
        Gets the credentials of this DiscoveryDetails.

        :return: The credentials of this DiscoveryDetails.
        :rtype: oci.stack_monitoring.models.CredentialCollection
        """
        return self._credentials

    @credentials.setter
    def credentials(self, credentials):
        """
        Sets the credentials of this DiscoveryDetails.

        :param credentials: The credentials of this DiscoveryDetails.
        :type: oci.stack_monitoring.models.CredentialCollection
        """
        self._credentials = credentials

    @property
    def tags(self):
        """
        Gets the tags of this DiscoveryDetails.

        :return: The tags of this DiscoveryDetails.
        :rtype: oci.stack_monitoring.models.PropertyDetails
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """
        Sets the tags of this DiscoveryDetails.

        :param tags: The tags of this DiscoveryDetails.
        :type: oci.stack_monitoring.models.PropertyDetails
        """
        self._tags = tags

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class DefaultPhaseTwoParameters(object):
    """
    Default phase two parameters.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new DefaultPhaseTwoParameters object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param default_encryption_algorithms:
            The value to assign to the default_encryption_algorithms property of this DefaultPhaseTwoParameters.
        :type default_encryption_algorithms: list[str]

        :param default_authentication_algorithms:
            The value to assign to the default_authentication_algorithms property of this DefaultPhaseTwoParameters.
        :type default_authentication_algorithms: list[str]

        :param default_pfs_dh_group:
            The value to assign to the default_pfs_dh_group property of this DefaultPhaseTwoParameters.
        :type default_pfs_dh_group: str

        """
        self.swagger_types = {
            'default_encryption_algorithms': 'list[str]',
            'default_authentication_algorithms': 'list[str]',
            'default_pfs_dh_group': 'str'
        }

        self.attribute_map = {
            'default_encryption_algorithms': 'defaultEncryptionAlgorithms',
            'default_authentication_algorithms': 'defaultAuthenticationAlgorithms',
            'default_pfs_dh_group': 'defaultPfsDhGroup'
        }

        self._default_encryption_algorithms = None
        self._default_authentication_algorithms = None
        self._default_pfs_dh_group = None

    @property
    def default_encryption_algorithms(self):
        """
        Gets the default_encryption_algorithms of this DefaultPhaseTwoParameters.
        Default phase two encryption algorithms.


        :return: The default_encryption_algorithms of this DefaultPhaseTwoParameters.
        :rtype: list[str]
        """
        return self._default_encryption_algorithms

    @default_encryption_algorithms.setter
    def default_encryption_algorithms(self, default_encryption_algorithms):
        """
        Sets the default_encryption_algorithms of this DefaultPhaseTwoParameters.
        Default phase two encryption algorithms.


        :param default_encryption_algorithms: The default_encryption_algorithms of this DefaultPhaseTwoParameters.
        :type: list[str]
        """
        self._default_encryption_algorithms = default_encryption_algorithms

    @property
    def default_authentication_algorithms(self):
        """
        Gets the default_authentication_algorithms of this DefaultPhaseTwoParameters.
        Default phase two authentication algorithms.


        :return: The default_authentication_algorithms of this DefaultPhaseTwoParameters.
        :rtype: list[str]
        """
        return self._default_authentication_algorithms

    @default_authentication_algorithms.setter
    def default_authentication_algorithms(self, default_authentication_algorithms):
        """
        Sets the default_authentication_algorithms of this DefaultPhaseTwoParameters.
        Default phase two authentication algorithms.


        :param default_authentication_algorithms: The default_authentication_algorithms of this DefaultPhaseTwoParameters.
        :type: list[str]
        """
        self._default_authentication_algorithms = default_authentication_algorithms

    @property
    def default_pfs_dh_group(self):
        """
        Gets the default_pfs_dh_group of this DefaultPhaseTwoParameters.
        Default perfect forward secrecy Diffie-Hellman groups.


        :return: The default_pfs_dh_group of this DefaultPhaseTwoParameters.
        :rtype: str
        """
        return self._default_pfs_dh_group

    @default_pfs_dh_group.setter
    def default_pfs_dh_group(self, default_pfs_dh_group):
        """
        Sets the default_pfs_dh_group of this DefaultPhaseTwoParameters.
        Default perfect forward secrecy Diffie-Hellman groups.


        :param default_pfs_dh_group: The default_pfs_dh_group of this DefaultPhaseTwoParameters.
        :type: str
        """
        self._default_pfs_dh_group = default_pfs_dh_group

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ProvisionAuditPolicyDetails(object):
    """
    Details for audit policy provisioning.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ProvisionAuditPolicyDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param is_data_safe_service_account_excluded:
            The value to assign to the is_data_safe_service_account_excluded property of this ProvisionAuditPolicyDetails.
        :type is_data_safe_service_account_excluded: bool

        :param provision_audit_conditions:
            The value to assign to the provision_audit_conditions property of this ProvisionAuditPolicyDetails.
        :type provision_audit_conditions: list[oci.data_safe.models.ProvisionAuditConditions]

        """
        self.swagger_types = {
            'is_data_safe_service_account_excluded': 'bool',
            'provision_audit_conditions': 'list[ProvisionAuditConditions]'
        }

        self.attribute_map = {
            'is_data_safe_service_account_excluded': 'isDataSafeServiceAccountExcluded',
            'provision_audit_conditions': 'provisionAuditConditions'
        }

        self._is_data_safe_service_account_excluded = None
        self._provision_audit_conditions = None

    @property
    def is_data_safe_service_account_excluded(self):
        """
        Gets the is_data_safe_service_account_excluded of this ProvisionAuditPolicyDetails.
        Option provided to users at the target to indicate whether the Data Safe service account has to be excluded while provisioning the audit policies.


        :return: The is_data_safe_service_account_excluded of this ProvisionAuditPolicyDetails.
        :rtype: bool
        """
        return self._is_data_safe_service_account_excluded

    @is_data_safe_service_account_excluded.setter
    def is_data_safe_service_account_excluded(self, is_data_safe_service_account_excluded):
        """
        Sets the is_data_safe_service_account_excluded of this ProvisionAuditPolicyDetails.
        Option provided to users at the target to indicate whether the Data Safe service account has to be excluded while provisioning the audit policies.


        :param is_data_safe_service_account_excluded: The is_data_safe_service_account_excluded of this ProvisionAuditPolicyDetails.
        :type: bool
        """
        self._is_data_safe_service_account_excluded = is_data_safe_service_account_excluded

    @property
    def provision_audit_conditions(self):
        """
        **[Required]** Gets the provision_audit_conditions of this ProvisionAuditPolicyDetails.
        The audit policy details for provisioning.


        :return: The provision_audit_conditions of this ProvisionAuditPolicyDetails.
        :rtype: list[oci.data_safe.models.ProvisionAuditConditions]
        """
        return self._provision_audit_conditions

    @provision_audit_conditions.setter
    def provision_audit_conditions(self, provision_audit_conditions):
        """
        Sets the provision_audit_conditions of this ProvisionAuditPolicyDetails.
        The audit policy details for provisioning.


        :param provision_audit_conditions: The provision_audit_conditions of this ProvisionAuditPolicyDetails.
        :type: list[oci.data_safe.models.ProvisionAuditConditions]
        """
        self._provision_audit_conditions = provision_audit_conditions

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class IPSecConnectionTunnelErrorDetails(object):
    """
    Details for an error on an IPSec tunnel.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new IPSecConnectionTunnelErrorDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this IPSecConnectionTunnelErrorDetails.
        :type id: str

        :param error_code:
            The value to assign to the error_code property of this IPSecConnectionTunnelErrorDetails.
        :type error_code: str

        :param error_description:
            The value to assign to the error_description property of this IPSecConnectionTunnelErrorDetails.
        :type error_description: str

        :param solution:
            The value to assign to the solution property of this IPSecConnectionTunnelErrorDetails.
        :type solution: str

        :param oci_resources_link:
            The value to assign to the oci_resources_link property of this IPSecConnectionTunnelErrorDetails.
        :type oci_resources_link: str

        :param timestamp:
            The value to assign to the timestamp property of this IPSecConnectionTunnelErrorDetails.
        :type timestamp: datetime

        """
        self.swagger_types = {
            'id': 'str',
            'error_code': 'str',
            'error_description': 'str',
            'solution': 'str',
            'oci_resources_link': 'str',
            'timestamp': 'datetime'
        }

        self.attribute_map = {
            'id': 'id',
            'error_code': 'errorCode',
            'error_description': 'errorDescription',
            'solution': 'solution',
            'oci_resources_link': 'ociResourcesLink',
            'timestamp': 'timestamp'
        }

        self._id = None
        self._error_code = None
        self._error_description = None
        self._solution = None
        self._oci_resources_link = None
        self._timestamp = None

    @property
    def id(self):
        """
        **[Required]** Gets the id of this IPSecConnectionTunnelErrorDetails.
        Unique ID generated for each error report.


        :return: The id of this IPSecConnectionTunnelErrorDetails.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this IPSecConnectionTunnelErrorDetails.
        Unique ID generated for each error report.


        :param id: The id of this IPSecConnectionTunnelErrorDetails.
        :type: str
        """
        self._id = id

    @property
    def error_code(self):
        """
        **[Required]** Gets the error_code of this IPSecConnectionTunnelErrorDetails.
        Unique code describes the error type.


        :return: The error_code of this IPSecConnectionTunnelErrorDetails.
        :rtype: str
        """
        return self._error_code

    @error_code.setter
    def error_code(self, error_code):
        """
        Sets the error_code of this IPSecConnectionTunnelErrorDetails.
        Unique code describes the error type.


        :param error_code: The error_code of this IPSecConnectionTunnelErrorDetails.
        :type: str
        """
        self._error_code = error_code

    @property
    def error_description(self):
        """
        **[Required]** Gets the error_description of this IPSecConnectionTunnelErrorDetails.
        A detailed description of the error.


        :return: The error_description of this IPSecConnectionTunnelErrorDetails.
        :rtype: str
        """
        return self._error_description

    @error_description.setter
    def error_description(self, error_description):
        """
        Sets the error_description of this IPSecConnectionTunnelErrorDetails.
        A detailed description of the error.


        :param error_description: The error_description of this IPSecConnectionTunnelErrorDetails.
        :type: str
        """
        self._error_description = error_description

    @property
    def solution(self):
        """
        **[Required]** Gets the solution of this IPSecConnectionTunnelErrorDetails.
        Resolution for the error.


        :return: The solution of this IPSecConnectionTunnelErrorDetails.
        :rtype: str
        """
        return self._solution

    @solution.setter
    def solution(self, solution):
        """
        Sets the solution of this IPSecConnectionTunnelErrorDetails.
        Resolution for the error.


        :param solution: The solution of this IPSecConnectionTunnelErrorDetails.
        :type: str
        """
        self._solution = solution

    @property
    def oci_resources_link(self):
        """
        **[Required]** Gets the oci_resources_link of this IPSecConnectionTunnelErrorDetails.
        Link to more Oracle resources or relevant documentation.


        :return: The oci_resources_link of this IPSecConnectionTunnelErrorDetails.
        :rtype: str
        """
        return self._oci_resources_link

    @oci_resources_link.setter
    def oci_resources_link(self, oci_resources_link):
        """
        Sets the oci_resources_link of this IPSecConnectionTunnelErrorDetails.
        Link to more Oracle resources or relevant documentation.


        :param oci_resources_link: The oci_resources_link of this IPSecConnectionTunnelErrorDetails.
        :type: str
        """
        self._oci_resources_link = oci_resources_link

    @property
    def timestamp(self):
        """
        **[Required]** Gets the timestamp of this IPSecConnectionTunnelErrorDetails.
        Timestamp when the error occurred.


        :return: The timestamp of this IPSecConnectionTunnelErrorDetails.
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """
        Sets the timestamp of this IPSecConnectionTunnelErrorDetails.
        Timestamp when the error occurred.


        :param timestamp: The timestamp of this IPSecConnectionTunnelErrorDetails.
        :type: datetime
        """
        self._timestamp = timestamp

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class VulnerabilityAuditConfiguration(object):
    """
    Configuration for a Vulnerability Audit
    type: object
    """

    def __init__(self, **kwargs):
        """
        Initializes a new VulnerabilityAuditConfiguration object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param max_permissible_cvss_v2_score:
            The value to assign to the max_permissible_cvss_v2_score property of this VulnerabilityAuditConfiguration.
        :type max_permissible_cvss_v2_score: float

        :param max_permissible_cvss_v3_score:
            The value to assign to the max_permissible_cvss_v3_score property of this VulnerabilityAuditConfiguration.
        :type max_permissible_cvss_v3_score: float

        :param exclusions:
            The value to assign to the exclusions property of this VulnerabilityAuditConfiguration.
        :type exclusions: list[str]

        """
        self.swagger_types = {
            'max_permissible_cvss_v2_score': 'float',
            'max_permissible_cvss_v3_score': 'float',
            'exclusions': 'list[str]'
        }

        self.attribute_map = {
            'max_permissible_cvss_v2_score': 'maxPermissibleCvssV2Score',
            'max_permissible_cvss_v3_score': 'maxPermissibleCvssV3Score',
            'exclusions': 'exclusions'
        }

        self._max_permissible_cvss_v2_score = None
        self._max_permissible_cvss_v3_score = None
        self._exclusions = None

    @property
    def max_permissible_cvss_v2_score(self):
        """
        Gets the max_permissible_cvss_v2_score of this VulnerabilityAuditConfiguration.
        The minimum CVSS V2 score for a vulnerability to be considered by the audit as failed.


        :return: The max_permissible_cvss_v2_score of this VulnerabilityAuditConfiguration.
        :rtype: float
        """
        return self._max_permissible_cvss_v2_score

    @max_permissible_cvss_v2_score.setter
    def max_permissible_cvss_v2_score(self, max_permissible_cvss_v2_score):
        """
        Sets the max_permissible_cvss_v2_score of this VulnerabilityAuditConfiguration.
        The minimum CVSS V2 score for a vulnerability to be considered by the audit as failed.


        :param max_permissible_cvss_v2_score: The max_permissible_cvss_v2_score of this VulnerabilityAuditConfiguration.
        :type: float
        """
        self._max_permissible_cvss_v2_score = max_permissible_cvss_v2_score

    @property
    def max_permissible_cvss_v3_score(self):
        """
        Gets the max_permissible_cvss_v3_score of this VulnerabilityAuditConfiguration.
        The minimum CVSS V3 score for a vulnerability to be considered by the audit as failed.


        :return: The max_permissible_cvss_v3_score of this VulnerabilityAuditConfiguration.
        :rtype: float
        """
        return self._max_permissible_cvss_v3_score

    @max_permissible_cvss_v3_score.setter
    def max_permissible_cvss_v3_score(self, max_permissible_cvss_v3_score):
        """
        Sets the max_permissible_cvss_v3_score of this VulnerabilityAuditConfiguration.
        The minimum CVSS V3 score for a vulnerability to be considered by the audit as failed.


        :param max_permissible_cvss_v3_score: The max_permissible_cvss_v3_score of this VulnerabilityAuditConfiguration.
        :type: float
        """
        self._max_permissible_cvss_v3_score = max_permissible_cvss_v3_score

    @property
    def exclusions(self):
        """
        Gets the exclusions of this VulnerabilityAuditConfiguration.
        Pattern to exclude artifacts from audit.


        :return: The exclusions of this VulnerabilityAuditConfiguration.
        :rtype: list[str]
        """
        return self._exclusions

    @exclusions.setter
    def exclusions(self, exclusions):
        """
        Sets the exclusions of this VulnerabilityAuditConfiguration.
        Pattern to exclude artifacts from audit.


        :param exclusions: The exclusions of this VulnerabilityAuditConfiguration.
        :type: list[str]
        """
        self._exclusions = exclusions

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from __future__ import absolute_import

from .authorize_subscription_payment_details import AuthorizeSubscriptionPaymentDetails
from .authorize_subscription_payment_receipt import AuthorizeSubscriptionPaymentReceipt
from .bill_to_address import BillToAddress
from .billing_address import BillingAddress
from .country import Country
from .credit_card_payment_detail import CreditCardPaymentDetail
from .credit_card_payment_option import CreditCardPaymentOption
from .currency import Currency
from .invoice import Invoice
from .invoice_collection import InvoiceCollection
from .invoice_line_collection import InvoiceLineCollection
from .invoice_line_summary import InvoiceLineSummary
from .invoice_summary import InvoiceSummary
from .merchant_defined_data import MerchantDefinedData
from .other_payment_detail import OtherPaymentDetail
from .pay_invoice_details import PayInvoiceDetails
from .pay_invoice_receipt import PayInvoiceReceipt
from .pay_subscription_details import PaySubscriptionDetails
from .pay_subscription_receipt import PaySubscriptionReceipt
from .payment_detail import PaymentDetail
from .payment_gateway import PaymentGateway
from .payment_option import PaymentOption
from .paypal_payment_detail import PaypalPaymentDetail
from .paypal_payment_option import PaypalPaymentOption
from .subscription import Subscription
from .subscription_collection import SubscriptionCollection
from .subscription_summary import SubscriptionSummary
from .tax_info import TaxInfo
from .update_subscription_details import UpdateSubscriptionDetails

# Maps type names to classes for osp_gateway services.
osp_gateway_type_mapping = {
    "AuthorizeSubscriptionPaymentDetails": AuthorizeSubscriptionPaymentDetails,
    "AuthorizeSubscriptionPaymentReceipt": AuthorizeSubscriptionPaymentReceipt,
    "BillToAddress": BillToAddress,
    "BillingAddress": BillingAddress,
    "Country": Country,
    "CreditCardPaymentDetail": CreditCardPaymentDetail,
    "CreditCardPaymentOption": CreditCardPaymentOption,
    "Currency": Currency,
    "Invoice": Invoice,
    "InvoiceCollection": InvoiceCollection,
    "InvoiceLineCollection": InvoiceLineCollection,
    "InvoiceLineSummary": InvoiceLineSummary,
    "InvoiceSummary": InvoiceSummary,
    "MerchantDefinedData": MerchantDefinedData,
    "OtherPaymentDetail": OtherPaymentDetail,
    "PayInvoiceDetails": PayInvoiceDetails,
    "PayInvoiceReceipt": PayInvoiceReceipt,
    "PaySubscriptionDetails": PaySubscriptionDetails,
    "PaySubscriptionReceipt": PaySubscriptionReceipt,
    "PaymentDetail": PaymentDetail,
    "PaymentGateway": PaymentGateway,
    "PaymentOption": PaymentOption,
    "PaypalPaymentDetail": PaypalPaymentDetail,
    "PaypalPaymentOption": PaypalPaymentOption,
    "Subscription": Subscription,
    "SubscriptionCollection": SubscriptionCollection,
    "SubscriptionSummary": SubscriptionSummary,
    "TaxInfo": TaxInfo,
    "UpdateSubscriptionDetails": UpdateSubscriptionDetails
}



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .connection import Connection
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ConnectionFromObjectStorage(Connection):
    """
    The connection details for an Oracle Object Storage data asset.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ConnectionFromObjectStorage object with values from keyword arguments. The default value of the :py:attr:`~oci.data_integration.models.ConnectionFromObjectStorage.model_type` attribute
        of this class is ``ORACLE_OBJECT_STORAGE_CONNECTION`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param model_type:
            The value to assign to the model_type property of this ConnectionFromObjectStorage.
            Allowed values for this property are: "ORACLE_ADWC_CONNECTION", "ORACLE_ATP_CONNECTION", "ORACLE_OBJECT_STORAGE_CONNECTION", "ORACLEDB_CONNECTION", "MYSQL_CONNECTION", "GENERIC_JDBC_CONNECTION", "BICC_CONNECTION", "AMAZON_S3_CONNECTION", "BIP_CONNECTION"
        :type model_type: str

        :param key:
            The value to assign to the key property of this ConnectionFromObjectStorage.
        :type key: str

        :param model_version:
            The value to assign to the model_version property of this ConnectionFromObjectStorage.
        :type model_version: str

        :param parent_ref:
            The value to assign to the parent_ref property of this ConnectionFromObjectStorage.
        :type parent_ref: oci.data_integration.models.ParentReference

        :param name:
            The value to assign to the name property of this ConnectionFromObjectStorage.
        :type name: str

        :param description:
            The value to assign to the description property of this ConnectionFromObjectStorage.
        :type description: str

        :param object_version:
            The value to assign to the object_version property of this ConnectionFromObjectStorage.
        :type object_version: int

        :param object_status:
            The value to assign to the object_status property of this ConnectionFromObjectStorage.
        :type object_status: int

        :param identifier:
            The value to assign to the identifier property of this ConnectionFromObjectStorage.
        :type identifier: str

        :param primary_schema:
            The value to assign to the primary_schema property of this ConnectionFromObjectStorage.
        :type primary_schema: oci.data_integration.models.Schema

        :param connection_properties:
            The value to assign to the connection_properties property of this ConnectionFromObjectStorage.
        :type connection_properties: list[oci.data_integration.models.ConnectionProperty]

        :param is_default:
            The value to assign to the is_default property of this ConnectionFromObjectStorage.
        :type is_default: bool

        :param metadata:
            The value to assign to the metadata property of this ConnectionFromObjectStorage.
        :type metadata: oci.data_integration.models.ObjectMetadata

        :param key_map:
            The value to assign to the key_map property of this ConnectionFromObjectStorage.
        :type key_map: dict(str, str)

        :param credential_file_content:
            The value to assign to the credential_file_content property of this ConnectionFromObjectStorage.
        :type credential_file_content: str

        :param user_id:
            The value to assign to the user_id property of this ConnectionFromObjectStorage.
        :type user_id: str

        :param finger_print:
            The value to assign to the finger_print property of this ConnectionFromObjectStorage.
        :type finger_print: str

        :param pass_phrase:
            The value to assign to the pass_phrase property of this ConnectionFromObjectStorage.
        :type pass_phrase: str

        """
        self.swagger_types = {
            'model_type': 'str',
            'key': 'str',
            'model_version': 'str',
            'parent_ref': 'ParentReference',
            'name': 'str',
            'description': 'str',
            'object_version': 'int',
            'object_status': 'int',
            'identifier': 'str',
            'primary_schema': 'Schema',
            'connection_properties': 'list[ConnectionProperty]',
            'is_default': 'bool',
            'metadata': 'ObjectMetadata',
            'key_map': 'dict(str, str)',
            'credential_file_content': 'str',
            'user_id': 'str',
            'finger_print': 'str',
            'pass_phrase': 'str'
        }

        self.attribute_map = {
            'model_type': 'modelType',
            'key': 'key',
            'model_version': 'modelVersion',
            'parent_ref': 'parentRef',
            'name': 'name',
            'description': 'description',
            'object_version': 'objectVersion',
            'object_status': 'objectStatus',
            'identifier': 'identifier',
            'primary_schema': 'primarySchema',
            'connection_properties': 'connectionProperties',
            'is_default': 'isDefault',
            'metadata': 'metadata',
            'key_map': 'keyMap',
            'credential_file_content': 'credentialFileContent',
            'user_id': 'userId',
            'finger_print': 'fingerPrint',
            'pass_phrase': 'passPhrase'
        }

        self._model_type = None
        self._key = None
        self._model_version = None
        self._parent_ref = None
        self._name = None
        self._description = None
        self._object_version = None
        self._object_status = None
        self._identifier = None
        self._primary_schema = None
        self._connection_properties = None
        self._is_default = None
        self._metadata = None
        self._key_map = None
        self._credential_file_content = None
        self._user_id = None
        self._finger_print = None
        self._pass_phrase = None
        self._model_type = 'ORACLE_OBJECT_STORAGE_CONNECTION'

    @property
    def credential_file_content(self):
        """
        Gets the credential_file_content of this ConnectionFromObjectStorage.
        The credential file content from an Oracle Object Storage wallet.


        :return: The credential_file_content of this ConnectionFromObjectStorage.
        :rtype: str
        """
        return self._credential_file_content

    @credential_file_content.setter
    def credential_file_content(self, credential_file_content):
        """
        Sets the credential_file_content of this ConnectionFromObjectStorage.
        The credential file content from an Oracle Object Storage wallet.


        :param credential_file_content: The credential_file_content of this ConnectionFromObjectStorage.
        :type: str
        """
        self._credential_file_content = credential_file_content

    @property
    def user_id(self):
        """
        Gets the user_id of this ConnectionFromObjectStorage.
        The OCI user OCID for the user to connect to.


        :return: The user_id of this ConnectionFromObjectStorage.
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """
        Sets the user_id of this ConnectionFromObjectStorage.
        The OCI user OCID for the user to connect to.


        :param user_id: The user_id of this ConnectionFromObjectStorage.
        :type: str
        """
        self._user_id = user_id

    @property
    def finger_print(self):
        """
        Gets the finger_print of this ConnectionFromObjectStorage.
        The fingerprint for the user.


        :return: The finger_print of this ConnectionFromObjectStorage.
        :rtype: str
        """
        return self._finger_print

    @finger_print.setter
    def finger_print(self, finger_print):
        """
        Sets the finger_print of this ConnectionFromObjectStorage.
        The fingerprint for the user.


        :param finger_print: The finger_print of this ConnectionFromObjectStorage.
        :type: str
        """
        self._finger_print = finger_print

    @property
    def pass_phrase(self):
        """
        Gets the pass_phrase of this ConnectionFromObjectStorage.
        The passphrase for the connection.


        :return: The pass_phrase of this ConnectionFromObjectStorage.
        :rtype: str
        """
        return self._pass_phrase

    @pass_phrase.setter
    def pass_phrase(self, pass_phrase):
        """
        Sets the pass_phrase of this ConnectionFromObjectStorage.
        The passphrase for the connection.


        :param pass_phrase: The pass_phrase of this ConnectionFromObjectStorage.
        :type: str
        """
        self._pass_phrase = pass_phrase

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ChildReferenceDetail(object):
    """
    References used in an application.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ChildReferenceDetail object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param key:
            The value to assign to the key property of this ChildReferenceDetail.
        :type key: str

        :param target_object:
            The value to assign to the target_object property of this ChildReferenceDetail.
        :type target_object: object

        """
        self.swagger_types = {
            'key': 'str',
            'target_object': 'object'
        }

        self.attribute_map = {
            'key': 'key',
            'target_object': 'targetObject'
        }

        self._key = None
        self._target_object = None

    @property
    def key(self):
        """
        Gets the key of this ChildReferenceDetail.
        The child reference key.


        :return: The key of this ChildReferenceDetail.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Sets the key of this ChildReferenceDetail.
        The child reference key.


        :param key: The key of this ChildReferenceDetail.
        :type: str
        """
        self._key = key

    @property
    def target_object(self):
        """
        Gets the target_object of this ChildReferenceDetail.
        The new reference object to use instead of the original reference. For example, this can be a connection reference.


        :return: The target_object of this ChildReferenceDetail.
        :rtype: object
        """
        return self._target_object

    @target_object.setter
    def target_object(self, target_object):
        """
        Sets the target_object of this ChildReferenceDetail.
        The new reference object to use instead of the original reference. For example, this can be a connection reference.


        :param target_object: The target_object of this ChildReferenceDetail.
        :type: object
        """
        self._target_object = target_object

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class AttachDataAssetInfo(object):
    """
    The attach DataAsset response.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new AttachDataAssetInfo object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param reference_info:
            The value to assign to the reference_info property of this AttachDataAssetInfo.
        :type reference_info: dict(str, ValidationResult)

        """
        self.swagger_types = {
            'reference_info': 'dict(str, ValidationResult)'
        }

        self.attribute_map = {
            'reference_info': 'referenceInfo'
        }

        self._reference_info = None

    @property
    def reference_info(self):
        """
        **[Required]** Gets the reference_info of this AttachDataAssetInfo.
        Map of DataAsset name as key and results as value


        :return: The reference_info of this AttachDataAssetInfo.
        :rtype: dict(str, ValidationResult)
        """
        return self._reference_info

    @reference_info.setter
    def reference_info(self, reference_info):
        """
        Sets the reference_info of this AttachDataAssetInfo.
        Map of DataAsset name as key and results as value


        :param reference_info: The reference_info of this AttachDataAssetInfo.
        :type: dict(str, ValidationResult)
        """
        self._reference_info = reference_info

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class RequestProtection(object):
    """
    Module that allows to enable OCI-managed protection capabilities for incoming HTTP requests.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new RequestProtection object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param rules:
            The value to assign to the rules property of this RequestProtection.
        :type rules: list[oci.waf.models.ProtectionRule]

        :param body_inspection_size_limit_in_bytes:
            The value to assign to the body_inspection_size_limit_in_bytes property of this RequestProtection.
        :type body_inspection_size_limit_in_bytes: int

        :param body_inspection_size_limit_exceeded_action_name:
            The value to assign to the body_inspection_size_limit_exceeded_action_name property of this RequestProtection.
        :type body_inspection_size_limit_exceeded_action_name: str

        """
        self.swagger_types = {
            'rules': 'list[ProtectionRule]',
            'body_inspection_size_limit_in_bytes': 'int',
            'body_inspection_size_limit_exceeded_action_name': 'str'
        }

        self.attribute_map = {
            'rules': 'rules',
            'body_inspection_size_limit_in_bytes': 'bodyInspectionSizeLimitInBytes',
            'body_inspection_size_limit_exceeded_action_name': 'bodyInspectionSizeLimitExceededActionName'
        }

        self._rules = None
        self._body_inspection_size_limit_in_bytes = None
        self._body_inspection_size_limit_exceeded_action_name = None

    @property
    def rules(self):
        """
        Gets the rules of this RequestProtection.
        Ordered list of ProtectionRules. Rules are executed in order of appearance in this array.
        ProtectionRules in this array can only use protection Capabilities of REQUEST_PROTECTION_CAPABILITY type.


        :return: The rules of this RequestProtection.
        :rtype: list[oci.waf.models.ProtectionRule]
        """
        return self._rules

    @rules.setter
    def rules(self, rules):
        """
        Sets the rules of this RequestProtection.
        Ordered list of ProtectionRules. Rules are executed in order of appearance in this array.
        ProtectionRules in this array can only use protection Capabilities of REQUEST_PROTECTION_CAPABILITY type.


        :param rules: The rules of this RequestProtection.
        :type: list[oci.waf.models.ProtectionRule]
        """
        self._rules = rules

    @property
    def body_inspection_size_limit_in_bytes(self):
        """
        Gets the body_inspection_size_limit_in_bytes of this RequestProtection.
        Maximum size of inspected HTTP message body in bytes. Actions to take if this limit is exceeded are defined in `bodyInspectionSizeLimitExceededActionName`.

        Body inspection maximum size allowed is defined with per-tenancy limit: 8192 bytes.


        :return: The body_inspection_size_limit_in_bytes of this RequestProtection.
        :rtype: int
        """
        return self._body_inspection_size_limit_in_bytes

    @body_inspection_size_limit_in_bytes.setter
    def body_inspection_size_limit_in_bytes(self, body_inspection_size_limit_in_bytes):
        """
        Sets the body_inspection_size_limit_in_bytes of this RequestProtection.
        Maximum size of inspected HTTP message body in bytes. Actions to take if this limit is exceeded are defined in `bodyInspectionSizeLimitExceededActionName`.

        Body inspection maximum size allowed is defined with per-tenancy limit: 8192 bytes.


        :param body_inspection_size_limit_in_bytes: The body_inspection_size_limit_in_bytes of this RequestProtection.
        :type: int
        """
        self._body_inspection_size_limit_in_bytes = body_inspection_size_limit_in_bytes

    @property
    def body_inspection_size_limit_exceeded_action_name(self):
        """
        Gets the body_inspection_size_limit_exceeded_action_name of this RequestProtection.
        References action by name from actions defined in WebAppFirewallPolicy. Executed if HTTP message
        body size exceeds limit set in field `bodyInspectionSizeLimitInBytes`.

        If this field is `null` HTTP message body will inspected up to `bodyInspectionSizeLimitInBytes` and the rest
        will not be inspected by Protection Capabilities.

        Allowed action types:
        * **RETURN_HTTP_RESPONSE** terminates further execution of modules and rules and returns defined HTTP response.


        :return: The body_inspection_size_limit_exceeded_action_name of this RequestProtection.
        :rtype: str
        """
        return self._body_inspection_size_limit_exceeded_action_name

    @body_inspection_size_limit_exceeded_action_name.setter
    def body_inspection_size_limit_exceeded_action_name(self, body_inspection_size_limit_exceeded_action_name):
        """
        Sets the body_inspection_size_limit_exceeded_action_name of this RequestProtection.
        References action by name from actions defined in WebAppFirewallPolicy. Executed if HTTP message
        body size exceeds limit set in field `bodyInspectionSizeLimitInBytes`.

        If this field is `null` HTTP message body will inspected up to `bodyInspectionSizeLimitInBytes` and the rest
        will not be inspected by Protection Capabilities.

        Allowed action types:
        * **RETURN_HTTP_RESPONSE** terminates further execution of modules and rules and returns defined HTTP response.


        :param body_inspection_size_limit_exceeded_action_name: The body_inspection_size_limit_exceeded_action_name of this RequestProtection.
        :type: str
        """
        self._body_inspection_size_limit_exceeded_action_name = body_inspection_size_limit_exceeded_action_name

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class WorkItemSummary(object):
    """
    The LCM work request for a JVM installation site.
    """

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "ACCEPTED"
    STATUS_ACCEPTED = "ACCEPTED"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "IN_PROGRESS"
    STATUS_IN_PROGRESS = "IN_PROGRESS"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "CANCELING"
    STATUS_CANCELING = "CANCELING"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "CANCELED"
    STATUS_CANCELED = "CANCELED"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "SUCCEEDED"
    STATUS_SUCCEEDED = "SUCCEEDED"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "NEEDS_ATTENTION"
    STATUS_NEEDS_ATTENTION = "NEEDS_ATTENTION"

    #: A constant which can be used with the status property of a WorkItemSummary.
    #: This constant has a value of "RETRYING"
    STATUS_RETRYING = "RETRYING"

    def __init__(self, **kwargs):
        """
        Initializes a new WorkItemSummary object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this WorkItemSummary.
        :type id: str

        :param work_request_id:
            The value to assign to the work_request_id property of this WorkItemSummary.
        :type work_request_id: str

        :param installation_site:
            The value to assign to the installation_site property of this WorkItemSummary.
        :type installation_site: oci.jms.models.InstallationSite

        :param status:
            The value to assign to the status property of this WorkItemSummary.
            Allowed values for this property are: "ACCEPTED", "IN_PROGRESS", "CANCELING", "CANCELED", "SUCCEEDED", "NEEDS_ATTENTION", "RETRYING", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type status: str

        :param retry_count:
            The value to assign to the retry_count property of this WorkItemSummary.
        :type retry_count: int

        :param time_last_updated:
            The value to assign to the time_last_updated property of this WorkItemSummary.
        :type time_last_updated: datetime

        """
        self.swagger_types = {
            'id': 'str',
            'work_request_id': 'str',
            'installation_site': 'InstallationSite',
            'status': 'str',
            'retry_count': 'int',
            'time_last_updated': 'datetime'
        }

        self.attribute_map = {
            'id': 'id',
            'work_request_id': 'workRequestId',
            'installation_site': 'installationSite',
            'status': 'status',
            'retry_count': 'retryCount',
            'time_last_updated': 'timeLastUpdated'
        }

        self._id = None
        self._work_request_id = None
        self._installation_site = None
        self._status = None
        self._retry_count = None
        self._time_last_updated = None

    @property
    def id(self):
        """
        **[Required]** Gets the id of this WorkItemSummary.
        The unique ID of ths work item.


        :return: The id of this WorkItemSummary.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this WorkItemSummary.
        The unique ID of ths work item.


        :param id: The id of this WorkItemSummary.
        :type: str
        """
        self._id = id

    @property
    def work_request_id(self):
        """
        **[Required]** Gets the work_request_id of this WorkItemSummary.
        The OCID of the work request created this work item.


        :return: The work_request_id of this WorkItemSummary.
        :rtype: str
        """
        return self._work_request_id

    @work_request_id.setter
    def work_request_id(self, work_request_id):
        """
        Sets the work_request_id of this WorkItemSummary.
        The OCID of the work request created this work item.


        :param work_request_id: The work_request_id of this WorkItemSummary.
        :type: str
        """
        self._work_request_id = work_request_id

    @property
    def installation_site(self):
        """
        **[Required]** Gets the installation_site of this WorkItemSummary.

        :return: The installation_site of this WorkItemSummary.
        :rtype: oci.jms.models.InstallationSite
        """
        return self._installation_site

    @installation_site.setter
    def installation_site(self, installation_site):
        """
        Sets the installation_site of this WorkItemSummary.

        :param installation_site: The installation_site of this WorkItemSummary.
        :type: oci.jms.models.InstallationSite
        """
        self._installation_site = installation_site

    @property
    def status(self):
        """
        **[Required]** Gets the status of this WorkItemSummary.
        The status of the work item.

        Allowed values for this property are: "ACCEPTED", "IN_PROGRESS", "CANCELING", "CANCELED", "SUCCEEDED", "NEEDS_ATTENTION", "RETRYING", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The status of this WorkItemSummary.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """
        Sets the status of this WorkItemSummary.
        The status of the work item.


        :param status: The status of this WorkItemSummary.
        :type: str
        """
        allowed_values = ["ACCEPTED", "IN_PROGRESS", "CANCELING", "CANCELED", "SUCCEEDED", "NEEDS_ATTENTION", "RETRYING"]
        if not value_allowed_none_or_none_sentinel(status, allowed_values):
            status = 'UNKNOWN_ENUM_VALUE'
        self._status = status

    @property
    def retry_count(self):
        """
        **[Required]** Gets the retry_count of this WorkItemSummary.
        Number of times this work item is retried.


        :return: The retry_count of this WorkItemSummary.
        :rtype: int
        """
        return self._retry_count

    @retry_count.setter
    def retry_count(self, retry_count):
        """
        Sets the retry_count of this WorkItemSummary.
        Number of times this work item is retried.


        :param retry_count: The retry_count of this WorkItemSummary.
        :type: int
        """
        self._retry_count = retry_count

    @property
    def time_last_updated(self):
        """
        Gets the time_last_updated of this WorkItemSummary.
        The date and time the work item was last updated. (formatted according to `RFC3339`__).

        __ https://datatracker.ietf.org/doc/html/rfc3339


        :return: The time_last_updated of this WorkItemSummary.
        :rtype: datetime
        """
        return self._time_last_updated

    @time_last_updated.setter
    def time_last_updated(self, time_last_updated):
        """
        Sets the time_last_updated of this WorkItemSummary.
        The date and time the work item was last updated. (formatted according to `RFC3339`__).

        __ https://datatracker.ietf.org/doc/html/rfc3339


        :param time_last_updated: The time_last_updated of this WorkItemSummary.
        :type: datetime
        """
        self._time_last_updated = time_last_updated

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class UpdateBudgetDetails(object):
    """
    The update budget details.
    """

    #: A constant which can be used with the processing_period_type property of a UpdateBudgetDetails.
    #: This constant has a value of "INVOICE"
    PROCESSING_PERIOD_TYPE_INVOICE = "INVOICE"

    #: A constant which can be used with the processing_period_type property of a UpdateBudgetDetails.
    #: This constant has a value of "MONTH"
    PROCESSING_PERIOD_TYPE_MONTH = "MONTH"

    #: A constant which can be used with the reset_period property of a UpdateBudgetDetails.
    #: This constant has a value of "MONTHLY"
    RESET_PERIOD_MONTHLY = "MONTHLY"

    def __init__(self, **kwargs):
        """
        Initializes a new UpdateBudgetDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param display_name:
            The value to assign to the display_name property of this UpdateBudgetDetails.
        :type display_name: str

        :param description:
            The value to assign to the description property of this UpdateBudgetDetails.
        :type description: str

        :param amount:
            The value to assign to the amount property of this UpdateBudgetDetails.
        :type amount: float

        :param budget_processing_period_start_offset:
            The value to assign to the budget_processing_period_start_offset property of this UpdateBudgetDetails.
        :type budget_processing_period_start_offset: int

        :param processing_period_type:
            The value to assign to the processing_period_type property of this UpdateBudgetDetails.
            Allowed values for this property are: "INVOICE", "MONTH"
        :type processing_period_type: str

        :param reset_period:
            The value to assign to the reset_period property of this UpdateBudgetDetails.
            Allowed values for this property are: "MONTHLY"
        :type reset_period: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this UpdateBudgetDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this UpdateBudgetDetails.
        :type defined_tags: dict(str, dict(str, object))

        """
        self.swagger_types = {
            'display_name': 'str',
            'description': 'str',
            'amount': 'float',
            'budget_processing_period_start_offset': 'int',
            'processing_period_type': 'str',
            'reset_period': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))'
        }

        self.attribute_map = {
            'display_name': 'displayName',
            'description': 'description',
            'amount': 'amount',
            'budget_processing_period_start_offset': 'budgetProcessingPeriodStartOffset',
            'processing_period_type': 'processingPeriodType',
            'reset_period': 'resetPeriod',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags'
        }

        self._display_name = None
        self._description = None
        self._amount = None
        self._budget_processing_period_start_offset = None
        self._processing_period_type = None
        self._reset_period = None
        self._freeform_tags = None
        self._defined_tags = None

    @property
    def display_name(self):
        """
        Gets the display_name of this UpdateBudgetDetails.
        The displayName of the budget. Avoid entering confidential information.


        :return: The display_name of this UpdateBudgetDetails.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this UpdateBudgetDetails.
        The displayName of the budget. Avoid entering confidential information.


        :param display_name: The display_name of this UpdateBudgetDetails.
        :type: str
        """
        self._display_name = display_name

    @property
    def description(self):
        """
        Gets the description of this UpdateBudgetDetails.
        The description of the budget.


        :return: The description of this UpdateBudgetDetails.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this UpdateBudgetDetails.
        The description of the budget.


        :param description: The description of this UpdateBudgetDetails.
        :type: str
        """
        self._description = description

    @property
    def amount(self):
        """
        Gets the amount of this UpdateBudgetDetails.
        The amount of the budget expressed as a whole number in the currency of the customer's rate card.


        :return: The amount of this UpdateBudgetDetails.
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """
        Sets the amount of this UpdateBudgetDetails.
        The amount of the budget expressed as a whole number in the currency of the customer's rate card.


        :param amount: The amount of this UpdateBudgetDetails.
        :type: float
        """
        self._amount = amount

    @property
    def budget_processing_period_start_offset(self):
        """
        Gets the budget_processing_period_start_offset of this UpdateBudgetDetails.
        The number of days offset from the first day of the month, at which the budget processing period starts. In months that have fewer days than this value, processing will begin on the last day of that month. For example, for a value of 12, processing starts every month on the 12th at midnight.


        :return: The budget_processing_period_start_offset of this UpdateBudgetDetails.
        :rtype: int
        """
        return self._budget_processing_period_start_offset

    @budget_processing_period_start_offset.setter
    def budget_processing_period_start_offset(self, budget_processing_period_start_offset):
        """
        Sets the budget_processing_period_start_offset of this UpdateBudgetDetails.
        The number of days offset from the first day of the month, at which the budget processing period starts. In months that have fewer days than this value, processing will begin on the last day of that month. For example, for a value of 12, processing starts every month on the 12th at midnight.


        :param budget_processing_period_start_offset: The budget_processing_period_start_offset of this UpdateBudgetDetails.
        :type: int
        """
        self._budget_processing_period_start_offset = budget_processing_period_start_offset

    @property
    def processing_period_type(self):
        """
        Gets the processing_period_type of this UpdateBudgetDetails.
        The type of the budget processing period. Valid values are INVOICE and MONTH.

        Allowed values for this property are: "INVOICE", "MONTH"


        :return: The processing_period_type of this UpdateBudgetDetails.
        :rtype: str
        """
        return self._processing_period_type

    @processing_period_type.setter
    def processing_period_type(self, processing_period_type):
        """
        Sets the processing_period_type of this UpdateBudgetDetails.
        The type of the budget processing period. Valid values are INVOICE and MONTH.


        :param processing_period_type: The processing_period_type of this UpdateBudgetDetails.
        :type: str
        """
        allowed_values = ["INVOICE", "MONTH"]
        if not value_allowed_none_or_none_sentinel(processing_period_type, allowed_values):
            raise ValueError(
                "Invalid value for `processing_period_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._processing_period_type = processing_period_type

    @property
    def reset_period(self):
        """
        Gets the reset_period of this UpdateBudgetDetails.
        The reset period for the budget.

        Allowed values for this property are: "MONTHLY"


        :return: The reset_period of this UpdateBudgetDetails.
        :rtype: str
        """
        return self._reset_period

    @reset_period.setter
    def reset_period(self, reset_period):
        """
        Sets the reset_period of this UpdateBudgetDetails.
        The reset period for the budget.


        :param reset_period: The reset_period of this UpdateBudgetDetails.
        :type: str
        """
        allowed_values = ["MONTHLY"]
        if not value_allowed_none_or_none_sentinel(reset_period, allowed_values):
            raise ValueError(
                "Invalid value for `reset_period`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._reset_period = reset_period

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this UpdateBudgetDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
        For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :return: The freeform_tags of this UpdateBudgetDetails.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this UpdateBudgetDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
        For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :param freeform_tags: The freeform_tags of this UpdateBudgetDetails.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this UpdateBudgetDetails.
        Defined tags for this resource. Each key is predefined and scoped to a namespace.
        For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :return: The defined_tags of this UpdateBudgetDetails.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this UpdateBudgetDetails.
        Defined tags for this resource. Each key is predefined and scoped to a namespace.
        For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :param defined_tags: The defined_tags of this UpdateBudgetDetails.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from __future__ import absolute_import

from oci._vendor import requests  # noqa: F401
from oci._vendor import six

from oci import retry, circuit_breaker  # noqa: F401
from oci.base_client import BaseClient
from oci.config import get_config_value_or_default, validate_config
from oci.signer import Signer
from oci.util import Sentinel, get_signer_from_authentication_type, AUTHENTICATION_TYPE_FIELD_NAME
from .models import osp_gateway_type_mapping
missing = Sentinel("Missing")


class InvoiceServiceClient(object):
    """
    This site describes all the Rest endpoints of OSP Gateway.
    """

    def __init__(self, config, **kwargs):
        """
        Creates a new service client

        :param dict config:
            Configuration keys and values as per `SDK and Tool Configuration <https://docs.cloud.oracle.com/Content/API/Concepts/sdkconfig.htm>`__.
            The :py:meth:`~oci.config.from_file` method can be used to load configuration from a file. Alternatively, a ``dict`` can be passed. You can validate_config
            the dict using :py:meth:`~oci.config.validate_config`

        :param str service_endpoint: (optional)
            The endpoint of the service to call using this client. For example ``https://iaas.us-ashburn-1.oraclecloud.com``. If this keyword argument is
            not provided then it will be derived using the region in the config parameter. You should only provide this keyword argument if you have an explicit
            need to specify a service endpoint.

        :param timeout: (optional)
            The connection and read timeouts for the client. The default values are connection timeout 10 seconds and read timeout 60 seconds. This keyword argument can be provided
            as a single float, in which case the value provided is used for both the read and connection timeouts, or as a tuple of two floats. If
            a tuple is provided then the first value is used as the connection timeout and the second value as the read timeout.
        :type timeout: float or tuple(float, float)

        :param signer: (optional)
            The signer to use when signing requests made by the service client. The default is to use a :py:class:`~oci.signer.Signer` based on the values
            provided in the config parameter.

            One use case for this parameter is for `Instance Principals authentication <https://docs.cloud.oracle.com/Content/Identity/Tasks/callingservicesfrominstances.htm>`__
            by passing an instance of :py:class:`~oci.auth.signers.InstancePrincipalsSecurityTokenSigner` as the value for this keyword argument
        :type signer: :py:class:`~oci.signer.AbstractBaseSigner`

        :param obj retry_strategy: (optional)
            A retry strategy to apply to all calls made by this service client (i.e. at the client level). There is no retry strategy applied by default.
            Retry strategies can also be applied at the operation level by passing a ``retry_strategy`` keyword argument as part of calling the operation.
            Any value provided at the operation level will override whatever is specified at the client level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. A convenience :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY`
            is also available. The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

        :param obj circuit_breaker_strategy: (optional)
            A circuit breaker strategy to apply to all calls made by this service client (i.e. at the client level).
            This client uses :py:data:`~oci.circuit_breaker.DEFAULT_CIRCUIT_BREAKER_STRATEGY` as default if no circuit breaker strategy is provided.
            The specifics of circuit breaker strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/circuit_breakers.html>`__.

        :param function circuit_breaker_callback: (optional)
            Callback function to receive any exceptions triggerred by the circuit breaker.

        :param allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this client should allow control characters in the response object. By default, the client will not
            allow control characters to be in the response object.
        """
        validate_config(config, signer=kwargs.get('signer'))
        if 'signer' in kwargs:
            signer = kwargs['signer']

        elif AUTHENTICATION_TYPE_FIELD_NAME in config:
            signer = get_signer_from_authentication_type(config)

        else:
            signer = Signer(
                tenancy=config["tenancy"],
                user=config["user"],
                fingerprint=config["fingerprint"],
                private_key_file_location=config.get("key_file"),
                pass_phrase=get_config_value_or_default(config, "pass_phrase"),
                private_key_content=config.get("key_content")
            )

        base_client_init_kwargs = {
            'regional_client': True,
            'service_endpoint': kwargs.get('service_endpoint'),
            'base_path': '/20191001',
            'service_endpoint_template': 'https://ospap.oracle.com',
            'skip_deserialization': kwargs.get('skip_deserialization', False),
            'circuit_breaker_strategy': kwargs.get('circuit_breaker_strategy', circuit_breaker.GLOBAL_CIRCUIT_BREAKER_STRATEGY)
        }
        if 'timeout' in kwargs:
            base_client_init_kwargs['timeout'] = kwargs.get('timeout')
        if base_client_init_kwargs.get('circuit_breaker_strategy') is None:
            base_client_init_kwargs['circuit_breaker_strategy'] = circuit_breaker.DEFAULT_CIRCUIT_BREAKER_STRATEGY
        if 'allow_control_chars' in kwargs:
            base_client_init_kwargs['allow_control_chars'] = kwargs.get('allow_control_chars')
        self.base_client = BaseClient("invoice_service", config, signer, osp_gateway_type_mapping, **base_client_init_kwargs)
        self.retry_strategy = kwargs.get('retry_strategy')
        self.circuit_breaker_callback = kwargs.get('circuit_breaker_callback')

    def download_pdf_content(self, osp_home_region, compartment_id, internal_invoice_id, **kwargs):
        """
        Returns an invoice in pdf format


        :param str osp_home_region: (required)
            The home region's public name of the logged in user.

        :param str compartment_id: (required)
            The `OCID`__ of the compartment.

            __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm

        :param str internal_invoice_id: (required)
            The identifier of the invoice.

        :param str opc_request_id: (optional)
            The unique Oracle-assigned identifier for the request. If you need to contact Oracle about a
            particular request, please provide the request ID.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :return: A :class:`~oci.response.Response` object with data of type stream
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.cloud.oracle.com/en-us/iaas/tools/python-sdk-examples/2.70.1/ospgateway/download_pdf_content.py.html>`__ to see an example of how to use download_pdf_content API.
        """
        resource_path = "/invoices/{internalInvoiceId}/actions/downloadPdfContent"
        method = "POST"
        operation_name = "download_pdf_content"
        api_reference_link = ""

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "retry_strategy",
            "opc_request_id"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                "download_pdf_content got unknown kwargs: {!r}".format(extra_kwargs))

        path_params = {
            "internalInvoiceId": internal_invoice_id
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError('Parameter {} cannot be None, whitespace or empty string'.format(k))

        query_params = {
            "ospHomeRegion": osp_home_region,
            "compartmentId": compartment_id
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/pdf",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="stream",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="stream",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)

    def get_invoice(self, osp_home_region, compartment_id, internal_invoice_id, **kwargs):
        """
        Returns an invoice by invoice id


        :param str osp_home_region: (required)
            The home region's public name of the logged in user.

        :param str compartment_id: (required)
            The `OCID`__ of the compartment.

            __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm

        :param str internal_invoice_id: (required)
            The identifier of the invoice.

        :param str opc_request_id: (optional)
            The unique Oracle-assigned identifier for the request. If you need to contact Oracle about a
            particular request, please provide the request ID.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.osp_gateway.models.Invoice`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.cloud.oracle.com/en-us/iaas/tools/python-sdk-examples/2.70.1/ospgateway/get_invoice.py.html>`__ to see an example of how to use get_invoice API.
        """
        resource_path = "/invoices/{internalInvoiceId}"
        method = "GET"
        operation_name = "get_invoice"
        api_reference_link = ""

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "retry_strategy",
            "opc_request_id"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                "get_invoice got unknown kwargs: {!r}".format(extra_kwargs))

        path_params = {
            "internalInvoiceId": internal_invoice_id
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError('Parameter {} cannot be None, whitespace or empty string'.format(k))

        query_params = {
            "ospHomeRegion": osp_home_region,
            "compartmentId": compartment_id
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="Invoice",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="Invoice",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)

    def list_invoice_lines(self, osp_home_region, compartment_id, internal_invoice_id, **kwargs):
        """
        Returns the invoice product list by invoice id


        :param str osp_home_region: (required)
            The home region's public name of the logged in user.

        :param str compartment_id: (required)
            The `OCID`__ of the compartment.

            __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm

        :param str internal_invoice_id: (required)
            The identifier of the invoice.

        :param str opc_request_id: (optional)
            The unique Oracle-assigned identifier for the request. If you need to contact Oracle about a
            particular request, please provide the request ID.

        :param str page: (optional)
            For list pagination. The value of the opc-next-page response header from the previous \"List\" call.

        :param int limit: (optional)
            For list pagination. The maximum number of results per page, or items to return in a paginated \"List\" call.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.osp_gateway.models.InvoiceLineCollection`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.cloud.oracle.com/en-us/iaas/tools/python-sdk-examples/2.70.1/ospgateway/list_invoice_lines.py.html>`__ to see an example of how to use list_invoice_lines API.
        """
        resource_path = "/invoices/{internalInvoiceId}/invoiceLines"
        method = "GET"
        operation_name = "list_invoice_lines"
        api_reference_link = ""

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "retry_strategy",
            "opc_request_id",
            "page",
            "limit"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                "list_invoice_lines got unknown kwargs: {!r}".format(extra_kwargs))

        path_params = {
            "internalInvoiceId": internal_invoice_id
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError('Parameter {} cannot be None, whitespace or empty string'.format(k))

        query_params = {
            "ospHomeRegion": osp_home_region,
            "compartmentId": compartment_id,
            "page": kwargs.get("page", missing),
            "limit": kwargs.get("limit", missing)
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="InvoiceLineCollection",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="InvoiceLineCollection",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)

    def list_invoices(self, osp_home_region, compartment_id, **kwargs):
        """
        Returns a list of invoices


        :param str osp_home_region: (required)
            The home region's public name of the logged in user.

        :param str compartment_id: (required)
            The `OCID`__ of the compartment.

            __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm

        :param str opc_request_id: (optional)
            The unique Oracle-assigned identifier for the request. If you need to contact Oracle about a
            particular request, please provide the request ID.

        :param str invoice_id: (optional)
            The invoice query param (not unique).

        :param list[str] type: (optional)
            A filter to only return resources that match the given type exactly.

            Allowed values are: "HARDWARE", "SUBSCRIPTION", "SUPPORT", "LICENSE", "EDUCATION", "CONSULTING", "SERVICE", "USAGE"

        :param str search_text: (optional)
            A filter to only return resources that match the given value.
            Looking for partial matches in the following fileds:
            Invoice No., Reference No. (plan number), Payment Ref, Total Amount(plan number), Balance Due(plan number)
            and Party/Customer Name

        :param datetime time_invoice_start: (optional)
            description: Start time (UTC) of the target invoice date range for which to fetch invoice data (inclusive).

        :param datetime time_invoice_end: (optional)
            description: End time (UTC) of the target invoice date range for which to fetch invoice data (exclusive).

        :param datetime time_payment_start: (optional)
            description: Start time (UTC) of the target payment date range for which to fetch invoice data (inclusive).

        :param datetime time_payment_end: (optional)
            description: End time (UTC) of the target payment date range for which to fetch invoice data (exclusive).

        :param list[str] status: (optional)
            A filter to only return resources that match one of the status elements.

            Allowed values are: "OPEN", "PAST_DUE", "PAYMENT_SUBMITTED", "CLOSED"

        :param str page: (optional)
            For list pagination. The value of the opc-next-page response header from the previous \"List\" call.

        :param int limit: (optional)
            For list pagination. The maximum number of results per page, or items to return in a paginated \"List\" call.

        :param str sort_by: (optional)
            The field to sort by. Only one field can be selected for sorting.

            Allowed values are: "INVOICE_NO", "REF_NO", "STATUS", "TYPE", "INVOICE_DATE", "DUE_DATE", "PAYM_REF", "TOTAL_AMOUNT", "BALANCE_DUE"

        :param str sort_order: (optional)
            The sort order to use (ascending or descending).

            Allowed values are: "ASC", "DESC"

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.osp_gateway.models.InvoiceCollection`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.cloud.oracle.com/en-us/iaas/tools/python-sdk-examples/2.70.1/ospgateway/list_invoices.py.html>`__ to see an example of how to use list_invoices API.
        """
        resource_path = "/invoices"
        method = "GET"
        operation_name = "list_invoices"
        api_reference_link = ""

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "retry_strategy",
            "opc_request_id",
            "invoice_id",
            "type",
            "search_text",
            "time_invoice_start",
            "time_invoice_end",
            "time_payment_start",
            "time_payment_end",
            "status",
            "page",
            "limit",
            "sort_by",
            "sort_order"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                "list_invoices got unknown kwargs: {!r}".format(extra_kwargs))

        if 'type' in kwargs:
            type_allowed_values = ["HARDWARE", "SUBSCRIPTION", "SUPPORT", "LICENSE", "EDUCATION", "CONSULTING", "SERVICE", "USAGE"]
            for type_item in kwargs['type']:
                if type_item not in type_allowed_values:
                    raise ValueError(
                        "Invalid value for `type`, must be one of {0}".format(type_allowed_values)
                    )

        if 'status' in kwargs:
            status_allowed_values = ["OPEN", "PAST_DUE", "PAYMENT_SUBMITTED", "CLOSED"]
            for status_item in kwargs['status']:
                if status_item not in status_allowed_values:
                    raise ValueError(
                        "Invalid value for `status`, must be one of {0}".format(status_allowed_values)
                    )

        if 'sort_by' in kwargs:
            sort_by_allowed_values = ["INVOICE_NO", "REF_NO", "STATUS", "TYPE", "INVOICE_DATE", "DUE_DATE", "PAYM_REF", "TOTAL_AMOUNT", "BALANCE_DUE"]
            if kwargs['sort_by'] not in sort_by_allowed_values:
                raise ValueError(
                    "Invalid value for `sort_by`, must be one of {0}".format(sort_by_allowed_values)
                )

        if 'sort_order' in kwargs:
            sort_order_allowed_values = ["ASC", "DESC"]
            if kwargs['sort_order'] not in sort_order_allowed_values:
                raise ValueError(
                    "Invalid value for `sort_order`, must be one of {0}".format(sort_order_allowed_values)
                )

        query_params = {
            "ospHomeRegion": osp_home_region,
            "compartmentId": compartment_id,
            "invoiceId": kwargs.get("invoice_id", missing),
            "type": self.base_client.generate_collection_format_param(kwargs.get("type", missing), 'multi'),
            "searchText": kwargs.get("search_text", missing),
            "timeInvoiceStart": kwargs.get("time_invoice_start", missing),
            "timeInvoiceEnd": kwargs.get("time_invoice_end", missing),
            "timePaymentStart": kwargs.get("time_payment_start", missing),
            "timePaymentEnd": kwargs.get("time_payment_end", missing),
            "status": self.base_client.generate_collection_format_param(kwargs.get("status", missing), 'multi'),
            "page": kwargs.get("page", missing),
            "limit": kwargs.get("limit", missing),
            "sortBy": kwargs.get("sort_by", missing),
            "sortOrder": kwargs.get("sort_order", missing)
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="InvoiceCollection",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="InvoiceCollection",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)

    def pay_invoice(self, osp_home_region, internal_invoice_id, compartment_id, pay_invoice_details, **kwargs):
        """
        Pay an invoice


        :param str osp_home_region: (required)
            The home region's public name of the logged in user.

        :param str internal_invoice_id: (required)
            The identifier of the invoice.

        :param str compartment_id: (required)
            The `OCID`__ of the compartment.

            __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm

        :param oci.osp_gateway.models.PayInvoiceDetails pay_invoice_details: (required)
            Invoice payment request.

        :param str if_match: (optional)
            For optimistic concurrency control. In the PUT or DELETE call
            for a resource, set the `if-match` parameter to the value of the
            etag from a previous GET or POST response for that resource.
            The resource will be updated or deleted only if the etag you
            provide matches the resource's current etag value.

        :param str opc_retry_token: (optional)
            For requests that are not idempotent (creates being the main place of interest), THE APIs should take a header
            called opc-retry-token to identify the customer desire across requests, to introduce some level of idempotency.

        :param str opc_request_id: (optional)
            The unique Oracle-assigned identifier for the request. If you need to contact Oracle about a
            particular request, please provide the request ID.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.osp_gateway.models.PayInvoiceReceipt`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.cloud.oracle.com/en-us/iaas/tools/python-sdk-examples/2.70.1/ospgateway/pay_invoice.py.html>`__ to see an example of how to use pay_invoice API.
        """
        resource_path = "/invoices/{internalInvoiceId}/actions/pay"
        method = "POST"
        operation_name = "pay_invoice"
        api_reference_link = ""

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "retry_strategy",
            "if_match",
            "opc_retry_token",
            "opc_request_id"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                "pay_invoice got unknown kwargs: {!r}".format(extra_kwargs))

        path_params = {
            "internalInvoiceId": internal_invoice_id
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError('Parameter {} cannot be None, whitespace or empty string'.format(k))

        query_params = {
            "ospHomeRegion": osp_home_region,
            "compartmentId": compartment_id
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "if-match": kwargs.get("if_match", missing),
            "opc-retry-token": kwargs.get("opc_retry_token", missing),
            "opc-request-id": kwargs.get("opc_request_id", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_retry_token_if_needed(header_params)
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=pay_invoice_details,
                response_type="PayInvoiceReceipt",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=pay_invoice_details,
                response_type="PayInvoiceReceipt",
                allow_control_chars=kwargs.get('allow_control_chars'),
                operation_name=operation_name,
                api_reference_link=api_reference_link)



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class VolumeGroupReplica(object):
    """
    An asynchronous replica of a volume group that can then be used to create a new volume group
    or recover a volume group. For more information, see `Volume Group Replication`__.

    To use any of the API operations, you must be authorized in an IAM policy. If you're not authorized,
    talk to an administrator. If you're an administrator who needs to write policies to give users access, see
    `Getting Started with Policies`__.

    **Warning:** Oracle recommends that you avoid using any confidential information when you
    supply string values using the API.

    __ https://docs.cloud.oracle.com/iaas/Content/Block/Concepts/volumegroupreplication.htm
    __ https://docs.cloud.oracle.com/iaas/Content/Identity/Concepts/policygetstarted.htm
    """

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "PROVISIONING"
    LIFECYCLE_STATE_PROVISIONING = "PROVISIONING"

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "AVAILABLE"
    LIFECYCLE_STATE_AVAILABLE = "AVAILABLE"

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "ACTIVATING"
    LIFECYCLE_STATE_ACTIVATING = "ACTIVATING"

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "TERMINATING"
    LIFECYCLE_STATE_TERMINATING = "TERMINATING"

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "TERMINATED"
    LIFECYCLE_STATE_TERMINATED = "TERMINATED"

    #: A constant which can be used with the lifecycle_state property of a VolumeGroupReplica.
    #: This constant has a value of "FAULTY"
    LIFECYCLE_STATE_FAULTY = "FAULTY"

    def __init__(self, **kwargs):
        """
        Initializes a new VolumeGroupReplica object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param availability_domain:
            The value to assign to the availability_domain property of this VolumeGroupReplica.
        :type availability_domain: str

        :param compartment_id:
            The value to assign to the compartment_id property of this VolumeGroupReplica.
        :type compartment_id: str

        :param defined_tags:
            The value to assign to the defined_tags property of this VolumeGroupReplica.
        :type defined_tags: dict(str, dict(str, object))

        :param display_name:
            The value to assign to the display_name property of this VolumeGroupReplica.
        :type display_name: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this VolumeGroupReplica.
        :type freeform_tags: dict(str, str)

        :param id:
            The value to assign to the id property of this VolumeGroupReplica.
        :type id: str

        :param lifecycle_state:
            The value to assign to the lifecycle_state property of this VolumeGroupReplica.
            Allowed values for this property are: "PROVISIONING", "AVAILABLE", "ACTIVATING", "TERMINATING", "TERMINATED", "FAULTY", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type lifecycle_state: str

        :param size_in_gbs:
            The value to assign to the size_in_gbs property of this VolumeGroupReplica.
        :type size_in_gbs: int

        :param volume_group_id:
            The value to assign to the volume_group_id property of this VolumeGroupReplica.
        :type volume_group_id: str

        :param time_created:
            The value to assign to the time_created property of this VolumeGroupReplica.
        :type time_created: datetime

        :param member_replicas:
            The value to assign to the member_replicas property of this VolumeGroupReplica.
        :type member_replicas: list[oci.core.models.MemberReplica]

        :param time_last_synced:
            The value to assign to the time_last_synced property of this VolumeGroupReplica.
        :type time_last_synced: datetime

        """
        self.swagger_types = {
            'availability_domain': 'str',
            'compartment_id': 'str',
            'defined_tags': 'dict(str, dict(str, object))',
            'display_name': 'str',
            'freeform_tags': 'dict(str, str)',
            'id': 'str',
            'lifecycle_state': 'str',
            'size_in_gbs': 'int',
            'volume_group_id': 'str',
            'time_created': 'datetime',
            'member_replicas': 'list[MemberReplica]',
            'time_last_synced': 'datetime'
        }

        self.attribute_map = {
            'availability_domain': 'availabilityDomain',
            'compartment_id': 'compartmentId',
            'defined_tags': 'definedTags',
            'display_name': 'displayName',
            'freeform_tags': 'freeformTags',
            'id': 'id',
            'lifecycle_state': 'lifecycleState',
            'size_in_gbs': 'sizeInGBs',
            'volume_group_id': 'volumeGroupId',
            'time_created': 'timeCreated',
            'member_replicas': 'memberReplicas',
            'time_last_synced': 'timeLastSynced'
        }

        self._availability_domain = None
        self._compartment_id = None
        self._defined_tags = None
        self._display_name = None
        self._freeform_tags = None
        self._id = None
        self._lifecycle_state = None
        self._size_in_gbs = None
        self._volume_group_id = None
        self._time_created = None
        self._member_replicas = None
        self._time_last_synced = None

    @property
    def availability_domain(self):
        """
        **[Required]** Gets the availability_domain of this VolumeGroupReplica.
        The availability domain of the volume group replica.


        :return: The availability_domain of this VolumeGroupReplica.
        :rtype: str
        """
        return self._availability_domain

    @availability_domain.setter
    def availability_domain(self, availability_domain):
        """
        Sets the availability_domain of this VolumeGroupReplica.
        The availability domain of the volume group replica.


        :param availability_domain: The availability_domain of this VolumeGroupReplica.
        :type: str
        """
        self._availability_domain = availability_domain

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this VolumeGroupReplica.
        The OCID of the compartment that contains the volume group replica.


        :return: The compartment_id of this VolumeGroupReplica.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this VolumeGroupReplica.
        The OCID of the compartment that contains the volume group replica.


        :param compartment_id: The compartment_id of this VolumeGroupReplica.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this VolumeGroupReplica.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :return: The defined_tags of this VolumeGroupReplica.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this VolumeGroupReplica.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :param defined_tags: The defined_tags of this VolumeGroupReplica.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    @property
    def display_name(self):
        """
        **[Required]** Gets the display_name of this VolumeGroupReplica.
        A user-friendly name. Does not have to be unique, and it's changeable.
        Avoid entering confidential information.


        :return: The display_name of this VolumeGroupReplica.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this VolumeGroupReplica.
        A user-friendly name. Does not have to be unique, and it's changeable.
        Avoid entering confidential information.


        :param display_name: The display_name of this VolumeGroupReplica.
        :type: str
        """
        self._display_name = display_name

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this VolumeGroupReplica.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :return: The freeform_tags of this VolumeGroupReplica.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this VolumeGroupReplica.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :param freeform_tags: The freeform_tags of this VolumeGroupReplica.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def id(self):
        """
        **[Required]** Gets the id of this VolumeGroupReplica.
        The OCID for the volume group replica.


        :return: The id of this VolumeGroupReplica.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this VolumeGroupReplica.
        The OCID for the volume group replica.


        :param id: The id of this VolumeGroupReplica.
        :type: str
        """
        self._id = id

    @property
    def lifecycle_state(self):
        """
        **[Required]** Gets the lifecycle_state of this VolumeGroupReplica.
        The current state of a volume group.

        Allowed values for this property are: "PROVISIONING", "AVAILABLE", "ACTIVATING", "TERMINATING", "TERMINATED", "FAULTY", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The lifecycle_state of this VolumeGroupReplica.
        :rtype: str
        """
        return self._lifecycle_state

    @lifecycle_state.setter
    def lifecycle_state(self, lifecycle_state):
        """
        Sets the lifecycle_state of this VolumeGroupReplica.
        The current state of a volume group.


        :param lifecycle_state: The lifecycle_state of this VolumeGroupReplica.
        :type: str
        """
        allowed_values = ["PROVISIONING", "AVAILABLE", "ACTIVATING", "TERMINATING", "TERMINATED", "FAULTY"]
        if not value_allowed_none_or_none_sentinel(lifecycle_state, allowed_values):
            lifecycle_state = 'UNKNOWN_ENUM_VALUE'
        self._lifecycle_state = lifecycle_state

    @property
    def size_in_gbs(self):
        """
        **[Required]** Gets the size_in_gbs of this VolumeGroupReplica.
        The aggregate size of the volume group replica in GBs.


        :return: The size_in_gbs of this VolumeGroupReplica.
        :rtype: int
        """
        return self._size_in_gbs

    @size_in_gbs.setter
    def size_in_gbs(self, size_in_gbs):
        """
        Sets the size_in_gbs of this VolumeGroupReplica.
        The aggregate size of the volume group replica in GBs.


        :param size_in_gbs: The size_in_gbs of this VolumeGroupReplica.
        :type: int
        """
        self._size_in_gbs = size_in_gbs

    @property
    def volume_group_id(self):
        """
        **[Required]** Gets the volume_group_id of this VolumeGroupReplica.
        The OCID of the source volume group.


        :return: The volume_group_id of this VolumeGroupReplica.
        :rtype: str
        """
        return self._volume_group_id

    @volume_group_id.setter
    def volume_group_id(self, volume_group_id):
        """
        Sets the volume_group_id of this VolumeGroupReplica.
        The OCID of the source volume group.


        :param volume_group_id: The volume_group_id of this VolumeGroupReplica.
        :type: str
        """
        self._volume_group_id = volume_group_id

    @property
    def time_created(self):
        """
        **[Required]** Gets the time_created of this VolumeGroupReplica.
        The date and time the volume group replica was created. Format defined by `RFC3339`__.

        __ https://tools.ietf.org/html/rfc3339


        :return: The time_created of this VolumeGroupReplica.
        :rtype: datetime
        """
        return self._time_created

    @time_created.setter
    def time_created(self, time_created):
        """
        Sets the time_created of this VolumeGroupReplica.
        The date and time the volume group replica was created. Format defined by `RFC3339`__.

        __ https://tools.ietf.org/html/rfc3339


        :param time_created: The time_created of this VolumeGroupReplica.
        :type: datetime
        """
        self._time_created = time_created

    @property
    def member_replicas(self):
        """
        **[Required]** Gets the member_replicas of this VolumeGroupReplica.
        Volume replicas within this volume group replica.


        :return: The member_replicas of this VolumeGroupReplica.
        :rtype: list[oci.core.models.MemberReplica]
        """
        return self._member_replicas

    @member_replicas.setter
    def member_replicas(self, member_replicas):
        """
        Sets the member_replicas of this VolumeGroupReplica.
        Volume replicas within this volume group replica.


        :param member_replicas: The member_replicas of this VolumeGroupReplica.
        :type: list[oci.core.models.MemberReplica]
        """
        self._member_replicas = member_replicas

    @property
    def time_last_synced(self):
        """
        **[Required]** Gets the time_last_synced of this VolumeGroupReplica.
        The date and time the volume group replica was last synced from the source volume group.
        Format defined by `RFC3339`__.

        __ https://tools.ietf.org/html/rfc3339


        :return: The time_last_synced of this VolumeGroupReplica.
        :rtype: datetime
        """
        return self._time_last_synced

    @time_last_synced.setter
    def time_last_synced(self, time_last_synced):
        """
        Sets the time_last_synced of this VolumeGroupReplica.
        The date and time the volume group replica was last synced from the source volume group.
        Format defined by `RFC3339`__.

        __ https://tools.ietf.org/html/rfc3339


        :param time_last_synced: The time_last_synced of this VolumeGroupReplica.
        :type: datetime
        """
        self._time_last_synced = time_last_synced

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .update_auto_scale_policy_details import UpdateAutoScalePolicyDetails
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class UpdateScheduleBasedVerticalScalingPolicyDetails(UpdateAutoScalePolicyDetails):
    """
    Update details of a schedule based vertical autoscaling policy.

    In a schedule-based autoscaling policy, an autoscaling action is triggered at the scheduled execution time.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new UpdateScheduleBasedVerticalScalingPolicyDetails object with values from keyword arguments. The default value of the :py:attr:`~oci.bds.models.UpdateScheduleBasedVerticalScalingPolicyDetails.policy_type` attribute
        of this class is ``SCHEDULE_BASED_VERTICAL_SCALING_POLICY`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param policy_type:
            The value to assign to the policy_type property of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :type policy_type: str

        :param timezone:
            The value to assign to the timezone property of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :type timezone: str

        :param schedule_details:
            The value to assign to the schedule_details property of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :type schedule_details: list[oci.bds.models.VerticalScalingScheduleDetails]

        """
        self.swagger_types = {
            'policy_type': 'str',
            'timezone': 'str',
            'schedule_details': 'list[VerticalScalingScheduleDetails]'
        }

        self.attribute_map = {
            'policy_type': 'policyType',
            'timezone': 'timezone',
            'schedule_details': 'scheduleDetails'
        }

        self._policy_type = None
        self._timezone = None
        self._schedule_details = None
        self._policy_type = 'SCHEDULE_BASED_VERTICAL_SCALING_POLICY'

    @property
    def timezone(self):
        """
        Gets the timezone of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        The time zone of the execution schedule, in IANA time zone database name format


        :return: The timezone of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :rtype: str
        """
        return self._timezone

    @timezone.setter
    def timezone(self, timezone):
        """
        Sets the timezone of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        The time zone of the execution schedule, in IANA time zone database name format


        :param timezone: The timezone of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :type: str
        """
        self._timezone = timezone

    @property
    def schedule_details(self):
        """
        Gets the schedule_details of this UpdateScheduleBasedVerticalScalingPolicyDetails.

        :return: The schedule_details of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :rtype: list[oci.bds.models.VerticalScalingScheduleDetails]
        """
        return self._schedule_details

    @schedule_details.setter
    def schedule_details(self, schedule_details):
        """
        Sets the schedule_details of this UpdateScheduleBasedVerticalScalingPolicyDetails.

        :param schedule_details: The schedule_details of this UpdateScheduleBasedVerticalScalingPolicyDetails.
        :type: list[oci.bds.models.VerticalScalingScheduleDetails]
        """
        self._schedule_details = schedule_details

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class Vulnerability(object):
    """
    A vulnerability is a weakness or error in an artifact.
    A vulnerability is a generalization of a CVE (every CVE is a vulnerability, but not every vulnerability has a CVE).
    """

    def __init__(self, **kwargs):
        """
        Initializes a new Vulnerability object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this Vulnerability.
        :type id: str

        :param cvss_v2_score:
            The value to assign to the cvss_v2_score property of this Vulnerability.
        :type cvss_v2_score: float

        :param cvss_v3_score:
            The value to assign to the cvss_v3_score property of this Vulnerability.
        :type cvss_v3_score: float

        :param is_ignored:
            The value to assign to the is_ignored property of this Vulnerability.
        :type is_ignored: bool

        """
        self.swagger_types = {
            'id': 'str',
            'cvss_v2_score': 'float',
            'cvss_v3_score': 'float',
            'is_ignored': 'bool'
        }

        self.attribute_map = {
            'id': 'id',
            'cvss_v2_score': 'cvssV2Score',
            'cvss_v3_score': 'cvssV3Score',
            'is_ignored': 'isIgnored'
        }

        self._id = None
        self._cvss_v2_score = None
        self._cvss_v3_score = None
        self._is_ignored = None

    @property
    def id(self):
        """
        **[Required]** Gets the id of this Vulnerability.
        Unique vulnerability identifier, e.g. CVE-1999-0067.


        :return: The id of this Vulnerability.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this Vulnerability.
        Unique vulnerability identifier, e.g. CVE-1999-0067.


        :param id: The id of this Vulnerability.
        :type: str
        """
        self._id = id

    @property
    def cvss_v2_score(self):
        """
        **[Required]** Gets the cvss_v2_score of this Vulnerability.
        Common Vulnerability Scoring System (CVSS) Version 2.


        :return: The cvss_v2_score of this Vulnerability.
        :rtype: float
        """
        return self._cvss_v2_score

    @cvss_v2_score.setter
    def cvss_v2_score(self, cvss_v2_score):
        """
        Sets the cvss_v2_score of this Vulnerability.
        Common Vulnerability Scoring System (CVSS) Version 2.


        :param cvss_v2_score: The cvss_v2_score of this Vulnerability.
        :type: float
        """
        self._cvss_v2_score = cvss_v2_score

    @property
    def cvss_v3_score(self):
        """
        **[Required]** Gets the cvss_v3_score of this Vulnerability.
        Common Vulnerability Scoring System (CVSS) Version 3.


        :return: The cvss_v3_score of this Vulnerability.
        :rtype: float
        """
        return self._cvss_v3_score

    @cvss_v3_score.setter
    def cvss_v3_score(self, cvss_v3_score):
        """
        Sets the cvss_v3_score of this Vulnerability.
        Common Vulnerability Scoring System (CVSS) Version 3.


        :param cvss_v3_score: The cvss_v3_score of this Vulnerability.
        :type: float
        """
        self._cvss_v3_score = cvss_v3_score

    @property
    def is_ignored(self):
        """
        **[Required]** Gets the is_ignored of this Vulnerability.
        Indicates if the vulnerability was ignored according to the audit configuration.


        :return: The is_ignored of this Vulnerability.
        :rtype: bool
        """
        return self._is_ignored

    @is_ignored.setter
    def is_ignored(self, is_ignored):
        """
        Sets the is_ignored of this Vulnerability.
        Indicates if the vulnerability was ignored according to the audit configuration.


        :param is_ignored: The is_ignored of this Vulnerability.
        :type: bool
        """
        self._is_ignored = is_ignored

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .format_entry import FormatEntry
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class RandomDateFormatEntry(FormatEntry):
    """
    The Random Date masking format generates random and unique dates within a range.
    The date range is defined by the startDate and endDate attributes. The start date
    must be less than or equal to the end date. When masking columns with uniqueness
    constraint, ensure that the date range is sufficient enough to generate unique
    values. To learn more, check Random Date in the Data Safe documentation.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new RandomDateFormatEntry object with values from keyword arguments. The default value of the :py:attr:`~oci.data_safe.models.RandomDateFormatEntry.type` attribute
        of this class is ``RANDOM_DATE`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param type:
            The value to assign to the type property of this RandomDateFormatEntry.
            Allowed values for this property are: "DELETE_ROWS", "DETERMINISTIC_SUBSTITUTION", "DETERMINISTIC_ENCRYPTION", "DETERMINISTIC_ENCRYPTION_DATE", "FIXED_NUMBER", "FIXED_STRING", "LIBRARY_MASKING_FORMAT", "NULL_VALUE", "POST_PROCESSING_FUNCTION", "PRESERVE_ORIGINAL_DATA", "RANDOM_DATE", "RANDOM_DECIMAL_NUMBER", "RANDOM_DIGITS", "RANDOM_LIST", "RANDOM_NUMBER", "RANDOM_STRING", "RANDOM_SUBSTITUTION", "REGULAR_EXPRESSION", "SHUFFLE", "SQL_EXPRESSION", "SUBSTRING", "TRUNCATE_TABLE", "USER_DEFINED_FUNCTION"
        :type type: str

        :param description:
            The value to assign to the description property of this RandomDateFormatEntry.
        :type description: str

        :param start_date:
            The value to assign to the start_date property of this RandomDateFormatEntry.
        :type start_date: datetime

        :param end_date:
            The value to assign to the end_date property of this RandomDateFormatEntry.
        :type end_date: datetime

        """
        self.swagger_types = {
            'type': 'str',
            'description': 'str',
            'start_date': 'datetime',
            'end_date': 'datetime'
        }

        self.attribute_map = {
            'type': 'type',
            'description': 'description',
            'start_date': 'startDate',
            'end_date': 'endDate'
        }

        self._type = None
        self._description = None
        self._start_date = None
        self._end_date = None
        self._type = 'RANDOM_DATE'

    @property
    def start_date(self):
        """
        **[Required]** Gets the start_date of this RandomDateFormatEntry.
        The lower bound of the range within which random dates should be generated.
        The start date must be less than or equal to the end date.


        :return: The start_date of this RandomDateFormatEntry.
        :rtype: datetime
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """
        Sets the start_date of this RandomDateFormatEntry.
        The lower bound of the range within which random dates should be generated.
        The start date must be less than or equal to the end date.


        :param start_date: The start_date of this RandomDateFormatEntry.
        :type: datetime
        """
        self._start_date = start_date

    @property
    def end_date(self):
        """
        **[Required]** Gets the end_date of this RandomDateFormatEntry.
        The upper bound of the range within which random dates should be generated.
        The end date must be greater than or equal to the start date.


        :return: The end_date of this RandomDateFormatEntry.
        :rtype: datetime
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """
        Sets the end_date of this RandomDateFormatEntry.
        The upper bound of the range within which random dates should be generated.
        The end date must be greater than or equal to the start date.


        :param end_date: The end_date of this RandomDateFormatEntry.
        :type: datetime
        """
        self._end_date = end_date

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
